@@ -22,6 +22,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/resource"
@@ -159,15 +160,10 @@ type ReconcileClusterClaim struct {
 
 // Reconcile reconciles a ClusterClaim.
 func (r *ReconcileClusterClaim) Reconcile(request reconcile.Request) (result reconcile.Result, returnErr error) {
-	start := time.Now()
-	logger := r.logger.WithField("clusterClaim", request.NamespacedName)
-
+	logger := controllerutils.BuildControllerLogger(ControllerName, "clusterClaim", request.NamespacedName)
 	logger.Infof("reconciling cluster claim")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		logger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the ClusterClaim instance
 	claim := &hivev1.ClusterClaim{}
@@ -311,10 +307,15 @@ func (r *ReconcileClusterClaim) cleanupResources(claim *hivev1.ClusterClaim, log
 	}
 
 	// Delete ClusterDeployment
-	if cd.DeletionTimestamp == nil {
+	toRemove := controllerutils.IsClaimedClusterMarkedForRemoval(cd)
+	if cd.DeletionTimestamp == nil && !toRemove {
 		logger.Info("deleting clusterDeployment")
-		if err := r.Delete(context.Background(), cd); err != nil {
-			logger.WithError(err).Log(controllerutils.LogLevel(err), "error deleting ClusterDeployment")
+		if cd.Annotations == nil {
+			cd.Annotations = map[string]string{}
+		}
+		cd.Annotations[constants.ClusterClaimRemoveClusterAnnotation] = "true"
+		if err := r.Update(context.Background(), cd); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "error updating ClusterDeployment to mark it for deletion")
 			return err
 		}
 	}
@@ -358,15 +359,45 @@ func (r *ReconcileClusterClaim) reconcileForExistingAssignment(claim *hivev1.Clu
 	if err := r.createRBAC(claim, cd, logger); err != nil {
 		return reconcile.Result{}, err
 	}
-	conds, changed := controllerutils.SetClusterClaimConditionWithChangeCheck(
-		claim.Status.Conditions,
+	var statusChanged bool
+	var changed bool
+	conds := claim.Status.Conditions
+
+	conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+		conds,
 		hivev1.ClusterClaimPendingCondition,
 		corev1.ConditionFalse,
 		"ClusterClaimed",
 		"Cluster claimed",
 		controllerutils.UpdateConditionIfReasonOrMessageChange,
 	)
-	if changed {
+	statusChanged = statusChanged || changed
+
+	hc := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition)
+	if hc == nil || hc.Status == corev1.ConditionFalse {
+		conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+			conds,
+			hivev1.ClusterRunningCondition,
+			corev1.ConditionTrue,
+			"Running",
+			"Cluster is running",
+			controllerutils.UpdateConditionIfReasonOrMessageChange,
+		)
+		statusChanged = statusChanged || changed
+	} else {
+		log.Debug("waiting for cluster to be running")
+		conds, changed = controllerutils.SetClusterClaimConditionWithChangeCheck(
+			conds,
+			hivev1.ClusterRunningCondition,
+			corev1.ConditionFalse,
+			"Resuming",
+			"Waiting for cluster to be running",
+			controllerutils.UpdateConditionIfReasonOrMessageChange,
+		)
+		statusChanged = statusChanged || changed
+	}
+	if statusChanged {
+		log.Debug("conditions changed, updating claim status")
 		claim.Status.Conditions = conds
 		if err := r.Status().Update(context.Background(), claim); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "could not update status of ClusterClaim")
