@@ -18,7 +18,6 @@ import (
 	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
-	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
@@ -57,6 +56,9 @@ const (
 	defaultRequeueTime = 10 * time.Second
 	maxProvisions      = 3
 
+	platformAuthFailureReason = "PlatformAuthError"
+	platformAuthSuccessReason = "PlatformAuthSuccess"
+
 	clusterImageSetNotFoundReason = "ClusterImageSetNotFound"
 	clusterImageSetFoundReason    = "ClusterImageSetFound"
 
@@ -64,17 +66,14 @@ const (
 	dnsReadyReason     = "DNSReady"
 	dnsReadyAnnotation = "hive.openshift.io/dnsready"
 
+	installAttemptsLimitReachedReason = "InstallAttemptsLimitReached"
+	installOnlyOnceSetReason          = "InstallOnlyOnceSet"
+	provisionNotStoppedReason         = "ProvisionNotStopped"
+
 	deleteAfterAnnotation    = "hive.openshift.io/delete-after" // contains a duration after which the cluster should be cleaned up.
 	tryInstallOnceAnnotation = "hive.openshift.io/try-install-once"
 
-	platformAWS       = "aws"
-	platformAzure     = "azure"
-	platformGCP       = "gcp"
-	platformOpenStack = "openstack"
-	platformVSphere   = "vsphere"
-	platformBaremetal = "baremetal"
-	platformUnknown   = "unknown"
-	regionUnknown     = "unknown"
+	regionUnknown = "unknown"
 )
 
 var (
@@ -159,10 +158,11 @@ func Add(mgr manager.Manager) error {
 // NewReconciler returns a new reconcile.Reconciler
 func NewReconciler(mgr manager.Manager, logger log.FieldLogger, rateLimiter flowcontrol.RateLimiter) reconcile.Reconciler {
 	r := &ReconcileClusterDeployment{
-		Client:       controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName, &rateLimiter),
-		scheme:       mgr.GetScheme(),
-		logger:       logger,
-		expectations: controllerutils.NewExpectations(logger),
+		Client:                                  controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName, &rateLimiter),
+		scheme:                                  mgr.GetScheme(),
+		logger:                                  logger,
+		expectations:                            controllerutils.NewExpectations(logger),
+		validateCredentialsForClusterDeployment: controllerutils.ValidateCredentialsForClusterDeployment,
 	}
 	r.remoteClusterAPIClientBuilder = func(cd *hivev1.ClusterDeployment) remoteclient.Builder {
 		return remoteclient.NewBuilder(r.Client, cd, ControllerName)
@@ -271,6 +271,10 @@ type ReconcileClusterDeployment struct {
 	// for the remote cluster's API server
 	remoteClusterAPIClientBuilder func(cd *hivev1.ClusterDeployment) remoteclient.Builder
 
+	// validateCredentialsForClusterDeployment is what this controller will call to validate
+	// that the platform creds are good (used for testing)
+	validateCredentialsForClusterDeployment func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error)
+
 	protectedDelete bool
 }
 
@@ -280,20 +284,10 @@ type ReconcileClusterDeployment struct {
 // Automatically generate RBAC rules to allow the Controller to read and write Deployments
 //
 func (r *ReconcileClusterDeployment) Reconcile(request reconcile.Request) (result reconcile.Result, returnErr error) {
-	start := time.Now()
-	cdLog := r.logger.WithFields(log.Fields{
-		"controller":        ControllerName,
-		"clusterDeployment": request.Name,
-		"namespace":         request.Namespace,
-	})
-
-	// For logging, we need to see when the reconciliation loop starts and ends.
+	cdLog := controllerutils.BuildControllerLogger(ControllerName, "clusterDeployment", request.NamespacedName)
 	cdLog.Info("reconciling cluster deployment")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		cdLog.WithField("elapsed", dur).WithField("result", result).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, cdLog)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the ClusterDeployment instance
 	cd := &hivev1.ClusterDeployment{}
@@ -460,11 +454,6 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 		return reconcile.Result{}, err
 	}
 
-	// Clear any provision underway metrics if we're installed or deleted.
-	if cd.Spec.Installed || cd.DeletionTimestamp != nil {
-		clearProvisionUnderwaySecondsMetric(cd, cdLog)
-	}
-
 	if cd.DeletionTimestamp != nil {
 		if !controllerutils.HasFinalizer(cd, hivev1.FinalizerDeprovision) {
 			// Make sure we have no deprovision underway metric even though this was probably cleared when we
@@ -593,12 +582,26 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 		return *result, err
 	}
 
-	// Indicate that the cluster is still installing:
-	hivemetrics.MetricClusterDeploymentProvisionUnderwaySeconds.WithLabelValues(
-		cd.Name,
-		cd.Namespace,
-		hivemetrics.GetClusterDeploymentType(cd)).Set(
-		time.Since(cd.CreationTimestamp.Time).Seconds())
+	// Sanity check the platform/cloud credentials.
+	validCreds, err := r.validatePlatformCreds(cd, cdLog)
+	if err != nil {
+		cdLog.WithError(err).Error("unable to validate platform credentials")
+		return reconcile.Result{}, err
+	}
+	// Make sure the condition is set properly.
+	_, err = r.setAuthenticationFailure(cd, validCreds, cdLog)
+	if err != nil {
+		cdLog.WithError(err).Error("unable to update clusterdeployment")
+		return reconcile.Result{}, err
+	}
+
+	// If the platform credentials are no good, return error and go into backoff
+	authCondition := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.AuthenticationFailureClusterDeploymentCondition)
+	if authCondition != nil && authCondition.Status == corev1.ConditionTrue {
+		authError := errors.New(authCondition.Message)
+		cdLog.WithError(authError).Error("cannot proceed with provision while platform credentials authentication is failing.")
+		return reconcile.Result{}, authError
+	}
 
 	imageSet, err := r.getClusterImageSet(cd, cdLog)
 	if err != nil {
@@ -638,10 +641,6 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 	}
 
 	if cd.Status.ProvisionRef == nil {
-		if cd.Status.InstallRestarts > 0 && cd.Annotations[tryInstallOnceAnnotation] == "true" {
-			cdLog.Debug("not creating new provision since the deployment is set to try install only once")
-			return reconcile.Result{}, nil
-		}
 		return r.startNewProvision(cd, releaseImage, cdLog)
 	}
 
@@ -681,6 +680,62 @@ func (r *ReconcileClusterDeployment) startNewProvision(
 		}
 	}
 
+	if cd.Status.InstallRestarts > 0 && cd.Annotations[tryInstallOnceAnnotation] == "true" {
+		cdLog.Debug("not creating new provision since the deployment is set to try install only once")
+		conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+			cd.Status.Conditions,
+			hivev1.ProvisionStoppedCondition,
+			corev1.ConditionTrue,
+			installOnlyOnceSetReason,
+			"Deployment is set to try install only once",
+			controllerutils.UpdateConditionIfReasonOrMessageChange)
+		if changed {
+			cd.Status.Conditions = conditions
+			cdLog.Debugf("setting ProvisionStoppedCondition to %v", corev1.ConditionTrue)
+			if err := r.Status().Update(context.TODO(), cd); err != nil {
+				cdLog.WithError(err).Log(controllerutils.LogLevel(err), "failed to update cluster deployment status")
+				return reconcile.Result{}, err
+			}
+		}
+		return reconcile.Result{}, nil
+	}
+	if cd.Spec.InstallAttemptsLimit != nil && cd.Status.InstallRestarts >= int(*cd.Spec.InstallAttemptsLimit) {
+		cdLog.Debug("not creating new provision since the install attempts limit has been reached")
+		conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+			cd.Status.Conditions,
+			hivev1.ProvisionStoppedCondition,
+			corev1.ConditionTrue,
+			installAttemptsLimitReachedReason,
+			"Install attempts limit reached",
+			controllerutils.UpdateConditionIfReasonOrMessageChange)
+		if changed {
+			cd.Status.Conditions = conditions
+			cdLog.Debugf("setting ProvisionStoppedCondition to %v", corev1.ConditionTrue)
+			if err := r.Status().Update(context.TODO(), cd); err != nil {
+				cdLog.WithError(err).Log(controllerutils.LogLevel(err), "failed to update cluster deployment status")
+				return reconcile.Result{}, err
+			}
+		}
+		return reconcile.Result{}, nil
+	}
+
+	conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+		cd.Status.Conditions,
+		hivev1.ProvisionStoppedCondition,
+		corev1.ConditionFalse,
+		provisionNotStoppedReason,
+		"Provision is not stopped",
+		controllerutils.UpdateConditionNever)
+	if changed {
+		cd.Status.Conditions = conditions
+		cdLog.Debugf("setting ProvisionStoppedCondition to %v", corev1.ConditionFalse)
+		if err := r.Status().Update(context.TODO(), cd); err != nil {
+			cdLog.WithError(err).Log(controllerutils.LogLevel(err), "failed to update cluster deployment status")
+			return reconcile.Result{}, err
+		}
+		return reconcile.Result{}, nil
+	}
+
 	if err := controllerutils.SetupClusterInstallServiceAccount(r, cd.Namespace, cdLog); err != nil {
 		cdLog.WithError(err).Log(controllerutils.LogLevel(err), "error setting up service account and role")
 		return reconcile.Result{}, err
@@ -694,20 +749,14 @@ func (r *ReconcileClusterDeployment) startNewProvision(
 	}
 	labels[constants.ClusterDeploymentNameLabel] = cd.Name
 
-	skipGatherLogs := os.Getenv(constants.SkipGatherLogsEnvVar) == "true"
-	if !skipGatherLogs {
-		if err := r.createPVC(cd, cdLog); err != nil {
-			return reconcile.Result{}, err
-		}
-	}
+	extraEnvVars := getInstallLogEnvVars(cd.Name)
 
 	podSpec, err := install.InstallerPodSpec(
 		cd,
 		provisionName,
 		releaseImage,
 		controllerutils.ServiceAccountName,
-		GetInstallLogsPVCName(cd),
-		skipGatherLogs,
+		extraEnvVars,
 	)
 	if err != nil {
 		cdLog.WithError(err).Error("could not generate installer pod spec")
@@ -743,6 +792,15 @@ func (r *ReconcileClusterDeployment) startNewProvision(
 		return reconcile.Result{}, err
 	}
 
+	if err := r.copyInstallLogSecret(provision.Namespace, extraEnvVars); err != nil {
+		if !apierrors.IsAlreadyExists(err) {
+			// Couldn't copy the install log secret for a reason other than it already exists.
+			// If the secret already exists, then we should just use that secret.
+			cdLog.WithError(err).Error("could not copy install log secret")
+			return reconcile.Result{}, err
+		}
+	}
+
 	r.expectations.ExpectCreations(types.NamespacedName{Namespace: cd.Namespace, Name: cd.Name}.String(), 1)
 	if err := r.Create(context.TODO(), provision); err != nil {
 		cdLog.WithError(err).Error("could not create provision")
@@ -761,6 +819,78 @@ func (r *ReconcileClusterDeployment) startNewProvision(
 	return reconcile.Result{}, nil
 }
 
+func (r *ReconcileClusterDeployment) copyInstallLogSecret(destNamespace string, extraEnvVars []corev1.EnvVar) error {
+	hiveNS := controllerutils.GetHiveNamespace()
+
+	srcSecretName, foundSrc := os.LookupEnv(constants.InstallLogsCredentialsSecretRefEnvVar)
+	if !foundSrc {
+		// If the src secret reference wasn't found, then don't attempt to copy the secret.
+		return nil
+	}
+
+	foundDest := false
+	var destSecretName string
+	for _, envVar := range extraEnvVars {
+		if envVar.Name == constants.InstallLogsCredentialsSecretRefEnvVar {
+			destSecretName = envVar.Value
+			foundDest = true
+		}
+	}
+
+	if !foundDest {
+		// If the dest secret reference wasn't found, then don't attempt to copy the secret.
+		return nil
+	}
+
+	src := types.NamespacedName{Name: srcSecretName, Namespace: hiveNS}
+	dest := types.NamespacedName{Name: destSecretName, Namespace: destNamespace}
+	return controllerutils.CopySecret(r, src, dest)
+}
+
+func getInstallLogEnvVars(secretPrefix string) []corev1.EnvVar {
+	extraEnvVars := []corev1.EnvVar{}
+
+	cloudProvider, found := os.LookupEnv(constants.InstallLogsUploadProviderEnvVar)
+	if !found {
+		return extraEnvVars
+	}
+
+	extraEnvVars = append(extraEnvVars, corev1.EnvVar{
+		Name:  constants.InstallLogsUploadProviderEnvVar,
+		Value: cloudProvider,
+	})
+
+	if cloudProvider == constants.InstallLogsUploadProviderAWS {
+		secretName, foundSrc := os.LookupEnv(constants.InstallLogsCredentialsSecretRefEnvVar)
+		if foundSrc {
+			extraEnvVars = append(extraEnvVars, corev1.EnvVar{
+				Name:  constants.InstallLogsCredentialsSecretRefEnvVar,
+				Value: secretPrefix + "-" + secretName,
+			})
+		}
+
+		extraEnvVars = addEnvVarIfFound(constants.InstallLogsAWSRegionEnvVar, extraEnvVars)
+		extraEnvVars = addEnvVarIfFound(constants.InstallLogsAWSServiceEndpointEnvVar, extraEnvVars)
+		extraEnvVars = addEnvVarIfFound(constants.InstallLogsAWSS3BucketEnvVar, extraEnvVars)
+	}
+
+	return extraEnvVars
+}
+
+func addEnvVarIfFound(name string, envVars []corev1.EnvVar) []corev1.EnvVar {
+	value, found := os.LookupEnv(name)
+	if !found {
+		return envVars
+	}
+
+	tmp := corev1.EnvVar{
+		Name:  name,
+		Value: value,
+	}
+
+	return append(envVars, tmp)
+}
+
 func (r *ReconcileClusterDeployment) reconcileExistingProvision(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) (result reconcile.Result, returnedErr error) {
 	cdLog = cdLog.WithField("provision", cd.Status.ProvisionRef.Name)
 	cdLog.Debug("reconciling existing provision")
@@ -952,70 +1082,6 @@ func (r *ReconcileClusterDeployment) clearOutCurrentProvision(cd *hivev1.Cluster
 	return reconcile.Result{}, nil
 }
 
-// GetInstallLogsPVCName returns the expected name of the persistent volume claim for cluster install failure logs.
-func GetInstallLogsPVCName(cd *hivev1.ClusterDeployment) string {
-	return apihelpers.GetResourceName(cd.Name, "install-logs")
-}
-
-// createPVC will create the PVC for the install logs if it does not already exist.
-func (r *ReconcileClusterDeployment) createPVC(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) error {
-
-	pvcName := GetInstallLogsPVCName(cd)
-
-	switch err := r.Get(context.TODO(), types.NamespacedName{Name: pvcName, Namespace: cd.Namespace}, &corev1.PersistentVolumeClaim{}); {
-	case err == nil:
-		cdLog.Debug("pvc already exists")
-		return nil
-	case !apierrors.IsNotFound(err):
-		cdLog.WithError(err).Error("error getting persistent volume claim")
-		return err
-	}
-
-	labels := map[string]string{
-		constants.InstallJobLabel:            "true",
-		constants.ClusterDeploymentNameLabel: cd.Name,
-	}
-	if cd.Labels != nil {
-		typeStr, ok := cd.Labels[hivev1.HiveClusterTypeLabel]
-		if ok {
-			labels[hivev1.HiveClusterTypeLabel] = typeStr
-		}
-	}
-
-	pvc := &corev1.PersistentVolumeClaim{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      pvcName,
-			Namespace: cd.Namespace,
-			Labels:    labels,
-		},
-		Spec: corev1.PersistentVolumeClaimSpec{
-			AccessModes: []corev1.PersistentVolumeAccessMode{
-				corev1.ReadWriteOnce,
-			},
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceStorage: resource.MustParse("1Gi"),
-				},
-			},
-		},
-	}
-
-	cdLog.WithField("pvc", pvc.Name).Info("creating persistent volume claim")
-
-	cdLog.WithField("derivedObject", pvc.Name).Debug("Setting labels on derived object")
-	pvc.Labels = k8slabels.AddLabel(pvc.Labels, constants.ClusterDeploymentNameLabel, cd.Name)
-	pvc.Labels = k8slabels.AddLabel(pvc.Labels, constants.PVCTypeLabel, constants.PVCTypeInstallLogs)
-	if err := controllerutil.SetControllerReference(cd, pvc, r.scheme); err != nil {
-		cdLog.WithError(err).Error("error setting controller reference on pvc")
-		return err
-	}
-	err := r.Create(context.TODO(), pvc)
-	if err != nil {
-		cdLog.WithError(err).Log(controllerutils.LogLevel(err), "error creating pvc")
-	}
-	return err
-}
-
 // getReleaseImage looks for a a release image in clusterdeployment or its corresponding imageset in the following order:
 // 1 - specified in the cluster deployment spec.images.releaseImage
 // 2 - referenced in the cluster deployment spec.imageSet
@@ -1059,6 +1125,11 @@ func (r *ReconcileClusterDeployment) statusUpdate(cd *hivev1.ClusterDeployment,
 	return err
 }
 
+const (
+	imagesResolvedReason = "ImagesResolved"
+	imagesResolvedMsg    = "Images required for cluster deployment installations are resolved"
+)
+
 func (r *ReconcileClusterDeployment) resolveInstallerImage(cd *hivev1.ClusterDeployment, imageSet *hivev1.ClusterImageSet, releaseImage string, cdLog log.FieldLogger) (*reconcile.Result, error) {
 	areImagesResolved := cd.Status.InstallerImage != nil && cd.Status.CLIImage != nil
 
@@ -1070,7 +1141,7 @@ func (r *ReconcileClusterDeployment) resolveInstallerImage(cd *hivev1.ClusterDep
 	// The job does not exist. If the images have been resolved, continue reconciling. Otherwise, create the job.
 	case apierrors.IsNotFound(err):
 		if areImagesResolved {
-			return nil, nil
+			return nil, r.setInstallImagesNotResolvedCondition(cd, corev1.ConditionFalse, imagesResolvedReason, imagesResolvedMsg, cdLog)
 		}
 
 		job := imageset.GenerateImageSetJob(cd, releaseImage, controllerutils.ServiceAccountName)
@@ -1109,7 +1180,7 @@ func (r *ReconcileClusterDeployment) resolveInstallerImage(cd *hivev1.ClusterDep
 	// If the images were not resolved, requeue and wait for the delete to complete.
 	case !existingJob.DeletionTimestamp.IsZero():
 		if areImagesResolved {
-			return nil, nil
+			return nil, r.setInstallImagesNotResolvedCondition(cd, corev1.ConditionFalse, imagesResolvedReason, imagesResolvedMsg, cdLog)
 		}
 		jobLog.Debug("imageset job is being deleted. Will recreate once deleted")
 		return &reconcile.Result{RequeueAfter: defaultRequeueTime}, err
@@ -1127,8 +1198,23 @@ func (r *ReconcileClusterDeployment) resolveInstallerImage(cd *hivev1.ClusterDep
 			return nil, err
 		}
 		if areImagesResolved {
-			return nil, nil
+			return nil, r.setInstallImagesNotResolvedCondition(cd, corev1.ConditionFalse, imagesResolvedReason, imagesResolvedMsg, cdLog)
 		}
+
+		// the job has failed to update the images and therefore
+		// we need to update the InstallImagesResolvedCondition to reflect why it failed.
+		for _, jcond := range existingJob.Status.Conditions {
+			if jcond.Type != batchv1.JobFailed {
+				continue
+			}
+
+			msg := fmt.Sprintf("The job %s/%s to resolve the image failed because of (%s) %s",
+				existingJob.Namespace, existingJob.Name,
+				jcond.Reason, jcond.Message,
+			)
+			return &reconcile.Result{}, r.setInstallImagesNotResolvedCondition(cd, corev1.ConditionTrue, "JobToResolveImagesFailed", msg, cdLog)
+		}
+
 		return &reconcile.Result{}, nil
 
 	// The job exists and is in progress. Wait for the job to finish before doing any more reconciliation.
@@ -1138,6 +1224,22 @@ func (r *ReconcileClusterDeployment) resolveInstallerImage(cd *hivev1.ClusterDep
 	}
 }
 
+func (r *ReconcileClusterDeployment) setInstallImagesNotResolvedCondition(cd *hivev1.ClusterDeployment, status corev1.ConditionStatus, reason string, message string, cdLog log.FieldLogger) error {
+	conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+		cd.Status.Conditions,
+		hivev1.InstallImagesNotResolvedCondition,
+		status,
+		reason,
+		message,
+		controllerutils.UpdateConditionIfReasonOrMessageChange)
+	if !changed {
+		return nil
+	}
+	cd.Status.Conditions = conditions
+	cdLog.Debugf("setting InstallImagesNotResolvedCondition to %v", status)
+	return r.Status().Update(context.TODO(), cd)
+}
+
 func (r *ReconcileClusterDeployment) setDNSNotReadyCondition(cd *hivev1.ClusterDeployment, status corev1.ConditionStatus, reason string, message string, cdLog log.FieldLogger) error {
 	conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
 		cd.Status.Conditions,
@@ -1154,6 +1256,36 @@ func (r *ReconcileClusterDeployment) setDNSNotReadyCondition(cd *hivev1.ClusterD
 	return r.Status().Update(context.TODO(), cd)
 }
 
+func (r *ReconcileClusterDeployment) setAuthenticationFailure(cd *hivev1.ClusterDeployment, authSuccessful bool, cdLog log.FieldLogger) (bool, error) {
+
+	var status corev1.ConditionStatus
+	var reason, message string
+
+	if authSuccessful {
+		status = corev1.ConditionFalse
+		reason = platformAuthSuccessReason
+		message = "Platform credentials passed authentication check"
+	} else {
+		status = corev1.ConditionTrue
+		reason = platformAuthFailureReason
+		message = "Platform credentials failed authentication check"
+	}
+
+	conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+		cd.Status.Conditions,
+		hivev1.AuthenticationFailureClusterDeploymentCondition,
+		status,
+		reason,
+		message,
+		controllerutils.UpdateConditionIfReasonOrMessageChange)
+	if !changed {
+		return false, nil
+	}
+	cd.Status.Conditions = conditions
+
+	return changed, r.Status().Update(context.TODO(), cd)
+}
+
 func (r *ReconcileClusterDeployment) setInstallLaunchErrorCondition(cd *hivev1.ClusterDeployment, status corev1.ConditionStatus, reason string, message string, cdLog log.FieldLogger) error {
 	conditions, changed := controllerutils.SetClusterDeploymentConditionWithChangeCheck(
 		cd.Status.Conditions,
@@ -1661,8 +1793,15 @@ func selectorPodWatchHandler(a handler.MapObject) []reconcile.Request {
 	return retval
 }
 
+// GetInstallLogsPVCName returns the expected name of the persistent volume claim for cluster install failure logs.
+// TODO: Remove this function and all calls to it. It's being left here for compatibility until the install log PVs are removed from all the installs.
+func GetInstallLogsPVCName(cd *hivev1.ClusterDeployment) string {
+	return apihelpers.GetResourceName(cd.Name, "install-logs")
+}
+
 // cleanupInstallLogPVC will immediately delete the PVC (should it exist) if the cluster was installed successfully, without retries.
 // If there were retries, it will delete the PVC if it has been more than 7 days since the job was completed.
+// TODO: Remove this function and all calls to it. It's being left here for compatibility until the install log PVs are removed from all the installs.
 func (r *ReconcileClusterDeployment) cleanupInstallLogPVC(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) error {
 	if !cd.Spec.Installed {
 		return nil
@@ -1727,7 +1866,6 @@ func generateDeprovision(cd *hivev1.ClusterDeployment) (*hivev1.ClusterDeprovisi
 	case cd.Spec.Platform.Azure != nil:
 		req.Spec.Platform.Azure = &hivev1.AzureClusterDeprovision{
 			CredentialsSecretRef: &cd.Spec.Platform.Azure.CredentialsSecretRef,
-			ResourceGroupName:    cd.Spec.Platform.Azure.ResourceGroupName,
 		}
 	case cd.Spec.Platform.GCP != nil:
 		req.Spec.Platform.GCP = &hivev1.GCPClusterDeprovision{
@@ -1736,8 +1874,9 @@ func generateDeprovision(cd *hivev1.ClusterDeployment) (*hivev1.ClusterDeprovisi
 		}
 	case cd.Spec.Platform.OpenStack != nil:
 		req.Spec.Platform.OpenStack = &hivev1.OpenStackClusterDeprovision{
-			Cloud:                cd.Spec.Platform.OpenStack.Cloud,
-			CredentialsSecretRef: &cd.Spec.Platform.OpenStack.CredentialsSecretRef,
+			Cloud:                 cd.Spec.Platform.OpenStack.Cloud,
+			CredentialsSecretRef:  &cd.Spec.Platform.OpenStack.CredentialsSecretRef,
+			CertificatesSecretRef: cd.Spec.Platform.OpenStack.CertificatesSecretRef,
 		}
 	case cd.Spec.Platform.VSphere != nil:
 		req.Spec.Platform.VSphere = &hivev1.VSphereClusterDeprovision{
@@ -1796,17 +1935,6 @@ func clearDeprovisionUnderwaySecondsMetric(cd *hivev1.ClusterDeployment, cdLog l
 	}
 }
 
-func clearProvisionUnderwaySecondsMetric(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) {
-	cleared := hivemetrics.MetricClusterDeploymentProvisionUnderwaySeconds.Delete(map[string]string{
-		"cluster_deployment": cd.Name,
-		"namespace":          cd.Namespace,
-		"cluster_type":       hivemetrics.GetClusterDeploymentType(cd),
-	})
-	if cleared {
-		cdLog.Debug("cleared metric: %v", hivemetrics.MetricClusterDeploymentProvisionUnderwaySeconds)
-	}
-}
-
 // initializeAnnotations() initializes the annotations if it is not already
 func initializeAnnotations(cd *hivev1.ClusterDeployment) {
 	if cd.Annotations == nil {
@@ -2006,6 +2134,11 @@ func (r *ReconcileClusterDeployment) deleteOldFailedProvisions(provs []*hivev1.C
 	}
 }
 
+// validatePlatformCreds ensure the platform/cloud credentials are at least good enough to authenticate with
+func (r *ReconcileClusterDeployment) validatePlatformCreds(cd *hivev1.ClusterDeployment, logger log.FieldLogger) (bool, error) {
+	return r.validateCredentialsForClusterDeployment(r.Client, cd, logger)
+}
+
 // checkForFailedSync returns true if it finds that the ClusterSync has the Failed condition set
 func checkForFailedSync(clusterSync *hiveintv1alpha1.ClusterSync) bool {
 	for _, cond := range clusterSync.Status.Conditions {
@@ -2106,19 +2239,19 @@ func (r *ReconcileClusterDeployment) addOwnershipToSecret(cd *hivev1.ClusterDepl
 func getClusterPlatform(cd *hivev1.ClusterDeployment) string {
 	switch {
 	case cd.Spec.Platform.AWS != nil:
-		return platformAWS
+		return constants.PlatformAWS
 	case cd.Spec.Platform.Azure != nil:
-		return platformAzure
+		return constants.PlatformAzure
 	case cd.Spec.Platform.GCP != nil:
-		return platformGCP
+		return constants.PlatformGCP
 	case cd.Spec.Platform.OpenStack != nil:
-		return platformOpenStack
+		return constants.PlatformOpenStack
 	case cd.Spec.Platform.VSphere != nil:
-		return platformVSphere
+		return constants.PlatformVSphere
 	case cd.Spec.Platform.BareMetal != nil:
-		return platformBaremetal
+		return constants.PlatformBaremetal
 	}
-	return platformUnknown
+	return constants.PlatformUnknown
 }
 
 // getClusterRegion returns the region of a given ClusterDeployment
