@@ -15,7 +15,7 @@ import (
 	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
-	"k8s.io/apimachinery/pkg/api/errors"
+
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
@@ -35,7 +35,6 @@ import (
 	"github.com/openshift/hive/pkg/apis"
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 	hivev1aws "github.com/openshift/hive/pkg/apis/hive/v1/aws"
-	hivev1azure "github.com/openshift/hive/pkg/apis/hive/v1/azure"
 	"github.com/openshift/hive/pkg/apis/hive/v1/baremetal"
 	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
@@ -56,6 +55,7 @@ const (
 	testSyncsetInstanceName = "testSSI"
 	metadataName            = "foo-lqmsh-metadata"
 	pullSecretSecret        = "pull-secret"
+	installLogSecret        = "install-log-secret"
 	globalPullSecret        = "global-pull-secret"
 	adminKubeconfigSecret   = "foo-lqmsh-admin-kubeconfig"
 	adminKubeconfig         = `clusters:
@@ -118,17 +118,17 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 	}
 
 	tests := []struct {
-		name                    string
-		existing                []runtime.Object
-		pendingCreation         bool
-		expectErr               bool
-		expectedRequeueAfter    time.Duration
-		expectPendingCreation   bool
-		expectConsoleRouteFetch bool
-		validate                func(client.Client, *testing.T)
-		reconcilerSetup         func(*ReconcileClusterDeployment)
+		name                          string
+		existing                      []runtime.Object
+		pendingCreation               bool
+		expectErr                     bool
+		expectedRequeueAfter          time.Duration
+		expectPendingCreation         bool
+		expectConsoleRouteFetch       bool
+		validate                      func(client.Client, *testing.T)
+		reconcilerSetup               func(*ReconcileClusterDeployment)
+		platformCredentialsValidation func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error)
 	}{
-
 		{
 			name: "Add finalizer",
 			existing: []runtime.Object{
@@ -153,12 +153,6 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 			validate: func(c client.Client, t *testing.T) {
 				provisions := getProvisions(c)
 				assert.Len(t, provisions, 1, "expected provision to exist")
-
-				pvc := &corev1.PersistentVolumeClaim{}
-				err := c.Get(context.TODO(), client.ObjectKey{Name: testInstallLogPVC().Name, Namespace: testNamespace}, pvc)
-				assert.NoError(t, err)
-				assert.Equal(t, testClusterDeployment().Name, pvc.Labels[constants.ClusterDeploymentNameLabel], "incorrect cluster deployment name label")
-				assert.Equal(t, constants.PVCTypeInstallLogs, pvc.Labels[constants.PVCTypeLabel], "incorrect pvc type label")
 			},
 		},
 		{
@@ -315,69 +309,6 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 				}
 			},
 		},
-		{
-			name: "PVC cleanup for successful install",
-			existing: []runtime.Object{
-				testInstalledClusterDeployment(time.Now()),
-				testInstallLogPVC(),
-				testMetadataConfigMap(),
-				testSecret(corev1.SecretTypeOpaque, adminKubeconfigSecret, "kubeconfig", adminKubeconfig),
-				testSecret(corev1.SecretTypeOpaque, adminPasswordSecret, "password", adminPassword),
-				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
-				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
-			},
-			validate: func(c client.Client, t *testing.T) {
-				pvc := &corev1.PersistentVolumeClaim{}
-				err := c.Get(context.TODO(), client.ObjectKey{Name: GetInstallLogsPVCName(testClusterDeployment()), Namespace: testNamespace}, pvc)
-				if assert.Error(t, err) {
-					assert.True(t, errors.IsNotFound(err))
-				}
-			},
-		},
-		{
-			name: "PVC preserved for install with restarts",
-			existing: []runtime.Object{
-				func() *hivev1.ClusterDeployment {
-					cd := testInstalledClusterDeployment(time.Now())
-					cd.Status.InstallRestarts = 5
-					return cd
-				}(),
-				testInstallLogPVC(),
-				testMetadataConfigMap(),
-				testSecret(corev1.SecretTypeOpaque, adminKubeconfigSecret, "kubeconfig", adminKubeconfig),
-				testSecret(corev1.SecretTypeOpaque, adminPasswordSecret, "password", adminPassword),
-				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
-				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
-			},
-			validate: func(c client.Client, t *testing.T) {
-				pvc := &corev1.PersistentVolumeClaim{}
-				err := c.Get(context.TODO(), client.ObjectKey{Name: testInstallLogPVC().Name, Namespace: testNamespace}, pvc)
-				assert.NoError(t, err)
-			},
-		},
-		{
-			name: "PVC cleanup for install with restarts after 7 days",
-			existing: []runtime.Object{
-				func() *hivev1.ClusterDeployment {
-					cd := testInstalledClusterDeployment(time.Now().Add(-8 * 24 * time.Hour))
-					cd.Status.InstallRestarts = 5
-					return cd
-				}(),
-				testInstallLogPVC(),
-				testMetadataConfigMap(),
-				testSecret(corev1.SecretTypeOpaque, adminKubeconfigSecret, "kubeconfig", adminKubeconfig),
-				testSecret(corev1.SecretTypeOpaque, adminPasswordSecret, "password", adminPassword),
-				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
-				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
-			},
-			validate: func(c client.Client, t *testing.T) {
-				pvc := &corev1.PersistentVolumeClaim{}
-				err := c.Get(context.TODO(), client.ObjectKey{Name: GetInstallLogsPVCName(testClusterDeployment()), Namespace: testNamespace}, pvc)
-				if assert.Error(t, err) {
-					assert.True(t, errors.IsNotFound(err))
-				}
-			},
-		},
 		{
 			name: "clusterdeployment must specify pull secret when there is no global pull secret ",
 			existing: []runtime.Object{
@@ -547,6 +478,54 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 				assert.Equal(t, constants.JobTypeImageSet, job.Labels[constants.JobTypeLabel], "incorrect job type label")
 			},
 		},
+		{
+			name: "failed image should set InstallImagesNotResolved condition on clusterdeployment",
+			existing: []runtime.Object{
+				func() *hivev1.ClusterDeployment {
+					cd := testClusterDeployment()
+					cd.Status.InstallerImage = nil
+					cd.Spec.Provisioning.ImageSetRef = &hivev1.ClusterImageSetReference{Name: testClusterImageSetName}
+					return cd
+				}(),
+				testClusterImageSet(),
+				testCompletedFailedImageSetJob(),
+				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
+				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
+			},
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				assertConditionStatus(t, cd, hivev1.InstallImagesNotResolvedCondition, corev1.ConditionTrue)
+				assertConditionReason(t, cd, hivev1.InstallImagesNotResolvedCondition, "JobToResolveImagesFailed")
+
+			},
+		},
+		{
+			name: "clear InstallImagesNotResolved condition on success",
+			existing: []runtime.Object{
+				func() *hivev1.ClusterDeployment {
+					cd := testClusterDeployment()
+					cd.Status.InstallerImage = pointer.StringPtr("test-installer-image")
+					cd.Status.CLIImage = pointer.StringPtr("test-cli-image")
+					cd.Spec.Provisioning.ImageSetRef = &hivev1.ClusterImageSetReference{Name: testClusterImageSetName}
+					cd.Status.Conditions = []hivev1.ClusterDeploymentCondition{{
+						Type:    hivev1.InstallImagesNotResolvedCondition,
+						Status:  corev1.ConditionTrue,
+						Reason:  "test-reason",
+						Message: "test-message",
+					}}
+					return cd
+				}(),
+				testClusterImageSet(),
+				testCompletedImageSetJob(),
+				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
+				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
+			},
+			expectPendingCreation: true,
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				assertConditionStatus(t, cd, hivev1.InstallImagesNotResolvedCondition, corev1.ConditionFalse)
+			},
+		},
 		{
 			name: "Delete imageset job when complete",
 			existing: []runtime.Object{
@@ -1445,41 +1424,107 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 			},
 		},
 		{
-			name: "set resourcegroupname for Azure deprovision",
+			name: "install attempts is less than the limit",
 			existing: []runtime.Object{
-				func() *hivev1.ClusterDeployment {
-					cd := testClusterDeploymentWithProvision()
-					cd.Spec.Platform.AWS = nil
-					cd.Spec.Platform.Azure = &hivev1azure.Platform{
-						CredentialsSecretRef: corev1.LocalObjectReference{
-							Name: "azure-credentials",
-						},
-						Region:            "centralus",
-						ResourceGroupName: "manual-resourcegroup",
-					}
-					cd.Labels[hivev1.HiveClusterPlatformLabel] = "azure"
-					cd.Labels[hivev1.HiveClusterRegionLabel] = "centralus"
-
-					now := metav1.Now()
-					cd.DeletionTimestamp = &now
-
+				func() runtime.Object {
+					cd := testClusterDeployment()
+					cd.Status.InstallRestarts = 1
+					cd.Spec.InstallAttemptsLimit = pointer.Int32Ptr(2)
 					return cd
 				}(),
 				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
+				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
+			},
+			expectPendingCreation: true,
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				require.NotNil(t, cd, "could not get ClusterDeployment")
+				assert.NotContains(t, cd.Status.Conditions, hivev1.ProvisionStoppedCondition, "ClusterDeployment should not have ProvisionStopped condition")
+			},
+		},
+		{
+			name: "install attempts is equal to the limit",
+			existing: []runtime.Object{
+				func() runtime.Object {
+					cd := testClusterDeployment()
+					cd.Status.InstallRestarts = 2
+					cd.Spec.InstallAttemptsLimit = pointer.Int32Ptr(2)
+					return cd
+				}(),
+				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
+				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
 			},
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				require.NotNil(t, cd, "could not get ClusterDeployment")
+				assertConditionStatus(t, cd, hivev1.ProvisionStoppedCondition, corev1.ConditionTrue)
+				assertConditionReason(t, cd, hivev1.ProvisionStoppedCondition, "InstallAttemptsLimitReached")
+			},
+		},
+		{
+			name: "install attempts is greater than the limit",
+			existing: []runtime.Object{
+				func() runtime.Object {
+					cd := testClusterDeployment()
+					cd.Status.InstallRestarts = 3
+					cd.Spec.InstallAttemptsLimit = pointer.Int32Ptr(2)
+					return cd
+				}(),
+				testSecret(corev1.SecretTypeDockerConfigJson, pullSecretSecret, corev1.DockerConfigJsonKey, "{}"),
+				testSecret(corev1.SecretTypeDockerConfigJson, constants.GetMergedPullSecretName(testClusterDeployment()), corev1.DockerConfigJsonKey, "{}"),
+			},
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				require.NotNil(t, cd, "could not get ClusterDeployment")
+				assertConditionStatus(t, cd, hivev1.ProvisionStoppedCondition, corev1.ConditionTrue)
+				assertConditionReason(t, cd, hivev1.ProvisionStoppedCondition, "InstallAttemptsLimitReached")
+			},
+		},
+		{
+			name: "auth condition when platform creds are bad",
+			existing: []runtime.Object{
+				testClusterDeployment(),
+			},
+			platformCredentialsValidation: func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error) {
+				return false, nil
+			},
+			expectErr: true,
 			validate: func(c client.Client, t *testing.T) {
 				cd := getCD(c)
 				require.NotNil(t, cd, "could not get ClusterDeployment")
 
-				deprovisionKey := types.NamespacedName{
-					Name:      cd.Name,
-					Namespace: cd.Namespace,
-				}
-				deprovision := &hivev1.ClusterDeprovision{}
-				err := c.Get(context.TODO(), deprovisionKey, deprovision)
-				require.NoError(t, err, "failed to get ClusterDeprovision")
+				assertConditionStatus(t, cd, hivev1.AuthenticationFailureClusterDeploymentCondition, corev1.ConditionTrue)
+			},
+		},
+		{
+			name: "no ClusterProvision when platform creds are bad",
+			existing: []runtime.Object{
+				func() *hivev1.ClusterDeployment {
+					cd := testClusterDeployment()
+					cd.Status.Conditions = []hivev1.ClusterDeploymentCondition{
+						{
+							Status:  corev1.ConditionTrue,
+							Type:    hivev1.AuthenticationFailureClusterDeploymentCondition,
+							Reason:  platformAuthFailureReason,
+							Message: "Platform credentials failed authentication check",
+						},
+					}
+					return cd
+				}(),
+			},
+			platformCredentialsValidation: func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error) {
+				return false, nil
+			},
+			expectErr: true,
+			validate: func(c client.Client, t *testing.T) {
+				cd := getCD(c)
+				require.NotNil(t, cd, "could not get ClusterDeployment")
 
-				assert.Equal(t, "manual-resourcegroup", deprovision.Spec.Platform.Azure.ResourceGroupName)
+				provisionList := &hivev1.ClusterProvisionList{}
+				err := c.List(context.TODO(), provisionList, client.InNamespace(cd.Namespace))
+				require.NoError(t, err, "unexpected error listing ClusterProvisions")
+
+				assert.Zero(t, len(provisionList.Items), "expected no ClusterProvision objects when platform creds are bad")
 			},
 		},
 	}
@@ -1491,13 +1536,21 @@ func TestClusterDeploymentReconcile(t *testing.T) {
 			controllerExpectations := controllerutils.NewExpectations(logger)
 			mockCtrl := gomock.NewController(t)
 			defer mockCtrl.Finish()
+
 			mockRemoteClientBuilder := remoteclientmock.NewMockBuilder(mockCtrl)
+
+			if test.platformCredentialsValidation == nil {
+				test.platformCredentialsValidation = func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error) {
+					return true, nil
+				}
+			}
 			rcd := &ReconcileClusterDeployment{
-				Client:                        fakeClient,
-				scheme:                        scheme.Scheme,
-				logger:                        logger,
-				expectations:                  controllerExpectations,
-				remoteClusterAPIClientBuilder: func(*hivev1.ClusterDeployment) remoteclient.Builder { return mockRemoteClientBuilder },
+				Client:                                  fakeClient,
+				scheme:                                  scheme.Scheme,
+				logger:                                  logger,
+				expectations:                            controllerExpectations,
+				remoteClusterAPIClientBuilder:           func(*hivev1.ClusterDeployment) remoteclient.Builder { return mockRemoteClientBuilder },
+				validateCredentialsForClusterDeployment: test.platformCredentialsValidation,
 			}
 
 			if test.reconcilerSetup != nil {
@@ -1938,13 +1991,6 @@ func testOldFailedProvision(time time.Time, attempt int) *hivev1.ClusterProvisio
 	return provision
 }
 
-func testInstallLogPVC() *corev1.PersistentVolumeClaim {
-	pvc := &corev1.PersistentVolumeClaim{}
-	pvc.Name = GetInstallLogsPVCName(testClusterDeployment())
-	pvc.Namespace = testNamespace
-	return pvc
-}
-
 func testMetadataConfigMap() *corev1.ConfigMap {
 	cm := &corev1.ConfigMap{}
 	cm.Name = metadataName
@@ -1959,11 +2005,15 @@ func testMetadataConfigMap() *corev1.ConfigMap {
 }
 
 func testSecret(secretType corev1.SecretType, name, key, value string) *corev1.Secret {
+	return testSecretWithNamespace(secretType, name, testNamespace, key, value)
+}
+
+func testSecretWithNamespace(secretType corev1.SecretType, name, namespace, key, value string) *corev1.Secret {
 	s := &corev1.Secret{
 		Type: secretType,
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
-			Namespace: testNamespace,
+			Namespace: namespace,
 		},
 		Data: map[string][]byte{
 			key: []byte(value),
@@ -2139,6 +2189,9 @@ func TestUpdatePullSecretInfo(t *testing.T) {
 				scheme:                        scheme.Scheme,
 				logger:                        log.WithField("controller", "clusterDeployment"),
 				remoteClusterAPIClientBuilder: func(*hivev1.ClusterDeployment) remoteclient.Builder { return mockRemoteClientBuilder },
+				validateCredentialsForClusterDeployment: func(client.Client, *hivev1.ClusterDeployment, log.FieldLogger) (bool, error) {
+					return true, nil
+				},
 			}
 
 			_, err := rcd.Reconcile(reconcile.Request{
@@ -2321,6 +2374,82 @@ func TestMergePullSecrets(t *testing.T) {
 	}
 }
 
+func TestCopyInstallLogSecret(t *testing.T) {
+	apis.AddToScheme(scheme.Scheme)
+
+	tests := []struct {
+		name                    string
+		existingObjs            []runtime.Object
+		existingEnvVars         []corev1.EnvVar
+		expectedErr             bool
+		expectedNumberOfSecrets int
+	}{
+		{
+			name: "copies secret",
+			existingObjs: []runtime.Object{
+				testSecretWithNamespace(corev1.SecretTypeOpaque, installLogSecret, "hive", "cloud", "cloudsecret"),
+			},
+			expectedNumberOfSecrets: 1,
+			existingEnvVars: []corev1.EnvVar{
+				{
+					Name:  constants.InstallLogsCredentialsSecretRefEnvVar,
+					Value: installLogSecret,
+				},
+			},
+		},
+		{
+			name:        "missing secret",
+			expectedErr: true,
+			existingEnvVars: []corev1.EnvVar{
+				{
+					Name:  constants.InstallLogsCredentialsSecretRefEnvVar,
+					Value: installLogSecret,
+				},
+			},
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			fakeClient := fake.NewFakeClient(test.existingObjs...)
+			mockCtrl := gomock.NewController(t)
+			defer mockCtrl.Finish()
+			mockRemoteClientBuilder := remoteclientmock.NewMockBuilder(mockCtrl)
+			rcd := &ReconcileClusterDeployment{
+				Client:                        fakeClient,
+				scheme:                        scheme.Scheme,
+				logger:                        log.WithField("controller", "clusterDeployment"),
+				remoteClusterAPIClientBuilder: func(*hivev1.ClusterDeployment) remoteclient.Builder { return mockRemoteClientBuilder },
+			}
+
+			for _, envVar := range test.existingEnvVars {
+				if err := os.Setenv(envVar.Name, envVar.Value); err == nil {
+					defer func() {
+						if err := os.Unsetenv(envVar.Name); err != nil {
+							t.Error(err)
+						}
+					}()
+				} else {
+					t.Error(err)
+				}
+			}
+
+			err := rcd.copyInstallLogSecret(testNamespace, test.existingEnvVars)
+			secretList := &corev1.SecretList{}
+			listErr := rcd.List(context.TODO(), secretList, client.InNamespace(testNamespace))
+
+			if test.expectedErr {
+				assert.Error(t, err)
+			} else {
+				assert.NoError(t, err)
+			}
+
+			assert.NoError(t, listErr, "Listing secrets returned an unexpected error")
+			assert.Equal(t, test.expectedNumberOfSecrets, len(secretList.Items), "Number of secrets different than expected")
+		})
+	}
+}
+
 func getProvisions(c client.Client) []*hivev1.ClusterProvision {
 	provisionList := &hivev1.ClusterProvisionList{}
 	if err := c.List(context.TODO(), provisionList); err != nil {
@@ -2347,3 +2476,20 @@ func testCompletedImageSetJob() *batchv1.Job {
 		},
 	}
 }
+
+func testCompletedFailedImageSetJob() *batchv1.Job {
+	return &batchv1.Job{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      imageSetJobName,
+			Namespace: testNamespace,
+		},
+		Status: batchv1.JobStatus{
+			Conditions: []batchv1.JobCondition{{
+				Type:    batchv1.JobFailed,
+				Status:  corev1.ConditionTrue,
+				Reason:  "ImagePullBackoff",
+				Message: "The pod failed to start because one the containers did not start",
+			}},
+		},
+	}
+}
