@@ -3,16 +3,19 @@ package clustersync
 import (
 	"context"
 	"fmt"
+	"math/big"
 	"math/rand"
 	"os"
 	"reflect"
 	"sort"
+	"strconv"
 	"strings"
 	"time"
 
 	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	log "github.com/sirupsen/logrus"
+	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -54,15 +57,17 @@ const (
 	labelCreateOnly        = "createOnly"
 	metricResultSuccess    = "success"
 	metricResultError      = "error"
+	stsName                = "hive-clustersync"
 )
 
 var (
-	metricTimeToApplySyncSet = prometheus.NewHistogram(
+	metricTimeToApplySyncSet = prometheus.NewHistogramVec(
 		prometheus.HistogramOpts{
 			Name:    "hive_syncset_apply_duration_seconds",
 			Help:    "Time to first successfully apply syncset to a cluster",
 			Buckets: []float64{5, 10, 30, 60, 300, 600, 1800, 3600},
 		},
+		[]string{"group"},
 	)
 	metricTimeToApplySelectorSyncSet = prometheus.NewHistogramVec(
 		prometheus.HistogramOpts{
@@ -120,6 +125,25 @@ func Add(mgr manager.Manager) error {
 	if err != nil {
 		return err
 	}
+
+	logger.Debug("Getting HIVE_CLUSTERSYNC_POD_NAME")
+	podname, found := os.LookupEnv("HIVE_CLUSTERSYNC_POD_NAME")
+
+	if !found {
+		return errors.New("environment variable HIVE_CLUSTERSYNC_POD_NAME not set")
+	}
+
+	logger.Debug("Setting ordinalID")
+	parts := strings.Split(podname, "-")
+	ordinalIDStr := parts[len(parts)-1]
+	ordinalID32, err := strconv.Atoi(ordinalIDStr)
+	if err != nil {
+		return errors.Wrap(err, "error converting ordinalID to int")
+	}
+
+	r.ordinalID = int64(ordinalID32)
+	logger.WithField("ordinalID", r.ordinalID).Debug("ordinalID set")
+
 	return AddToManager(mgr, r, concurrentReconciles, queueRateLimiter)
 }
 
@@ -148,8 +172,11 @@ func NewReconciler(mgr manager.Manager, rateLimiter flowcontrol.RateLimiter) (*R
 	}, nil
 }
 
-func resourceHelperBuilderFunc(restConfig *rest.Config, logger log.FieldLogger) (resource.Helper, error) {
-	return resource.NewHelperFromRESTConfig(restConfig, logger), nil
+func resourceHelperBuilderFunc(restConfig *rest.Config, fakeCluster bool, logger log.FieldLogger) (resource.Helper, error) {
+	if fakeCluster {
+		return resource.NewFakeHelper(logger), nil
+	}
+	return resource.NewHelperFromRESTConfig(restConfig, logger)
 }
 
 // AddToManager adds a new Controller to mgr with r as the reconcile.Reconciler
@@ -239,25 +266,82 @@ type ReconcileClusterSync struct {
 	logger          log.FieldLogger
 	reapplyInterval time.Duration
 
-	resourceHelperBuilder func(*rest.Config, log.FieldLogger) (resource.Helper, error)
+	resourceHelperBuilder func(*rest.Config, bool, log.FieldLogger) (resource.Helper, error)
 
 	// remoteClusterAPIClientBuilder is a function pointer to the function that gets a builder for building a client
 	// for the remote cluster's API server
 	remoteClusterAPIClientBuilder func(cd *hivev1.ClusterDeployment) remoteclient.Builder
+
+	ordinalID int64
+}
+
+func (r *ReconcileClusterSync) getAndCheckClusterSyncStatefulSet(logger log.FieldLogger) (*appsv1.StatefulSet, error) {
+	hiveNS := controllerutils.GetHiveNamespace()
+
+	logger.Debug("Getting statefulset")
+	sts := &appsv1.StatefulSet{}
+	err := r.Get(context.Background(), types.NamespacedName{Namespace: hiveNS, Name: stsName}, sts)
+	if err != nil {
+		logger.WithError(err).WithField("hiveNS", hiveNS).Error("error getting statefulset.")
+		return nil, err
+	}
+
+	logger.Debug("Ensuring replicas is set")
+	if sts.Spec.Replicas == nil {
+		return nil, errors.New("sts.Spec.Replicas not set")
+	}
+
+	if sts.Status.CurrentReplicas != *sts.Spec.Replicas {
+		// This ensures that we don't have partial syncing which may make it seem like things are working.
+		// TODO: Remove this once https://issues.redhat.com/browse/CO-1268 is completed as this should no longer be needed.
+		return nil, fmt.Errorf("statefulset replica count is off. current: %v  expected: %v", sts.Status.CurrentReplicas, *sts.Spec.Replicas)
+	}
+
+	// All checks passed
+	return sts, nil
+}
+
+// isSyncAssignedToMe determines if this instance of the controller is assigned to the resource being sync'd
+func (r *ReconcileClusterSync) isSyncAssignedToMe(sts *appsv1.StatefulSet, cd *hivev1.ClusterDeployment, logger log.FieldLogger) (bool, error) {
+	logger.Debug("Getting uid for hashing")
+	var uidAsBigInt big.Int
+
+	// There are a couple of assumptions here:
+	// * the clusterdeployment uid has 4 sections separated by hyphens
+	// * the 4 sections are hex numbers
+	// These assumptions are based on the fact that Kubernetes says UIDs are actually
+	// ISO/IEC 9834-8 UUIDs. If this changes, this code may fail and may need to be updated.
+	// https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#uids
+	hexUID := strings.Replace(string(cd.UID), "-", "", 4)
+	logger.Debugf("hexUID: %+v", hexUID)
+	uidAsBigInt.SetString(hexUID, 16)
+
+	logger.Debug("calculating replicas")
+	replicas := int64(*sts.Spec.Replicas)
+
+	logger.Debug("determining who is assigned to sync this cluster")
+	ordinalIDOfAssignee := uidAsBigInt.Mod(&uidAsBigInt, big.NewInt(replicas)).Int64()
+	assignedToMe := ordinalIDOfAssignee == r.ordinalID
+
+	logger.WithFields(log.Fields{
+		"replicas":            replicas,
+		"ordinalIDOfAssignee": ordinalIDOfAssignee,
+		"ordinalID":           r.ordinalID,
+		"assignedToMe":        assignedToMe,
+	}).Debug("computed values")
+
+	return assignedToMe, nil
 }
 
 // Reconcile reads the state of the ClusterDeployment and applies any SyncSets or SelectorSyncSets that need to be
 // applied or re-applied.
 func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	logger := r.logger.WithField("ClusterDeployment", request.NamespacedName)
-
+	logger := controllerutils.BuildControllerLogger(ControllerName, "clusterDeployment", request.NamespacedName)
 	logger.Infof("reconciling ClusterDeployment")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		logger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
+	defer recobsrv.ObserveControllerReconcileTime()
+
+	recobsrv.SetOutcome(hivemetrics.ReconcileOutcomeNoOp)
 
 	// Fetch the ClusterDeployment instance
 	cd := &hivev1.ClusterDeployment{}
@@ -271,8 +355,24 @@ func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.R
 		return reconcile.Result{}, err
 	}
 
-	// Is syncing paused?
-	if !controllerutils.ShouldSyncCluster(cd, logger) {
+	sts, err := r.getAndCheckClusterSyncStatefulSet(logger)
+	if err != nil {
+		log.WithError(err).Error("failed getting clustersync statefulset")
+		return reconcile.Result{}, err
+	}
+
+	if me, err := r.isSyncAssignedToMe(sts, cd, logger); !me || err != nil {
+		if err != nil {
+			logger.WithError(err).Error("failed determining which instance is assigned to sync this cluster")
+			return reconcile.Result{}, err
+		}
+
+		logger.Debug("not syncing because isSyncAssignedToMe returned false")
+		recobsrv.SetOutcome(hivemetrics.ReconcileOutcomeSkippedSync)
+		return reconcile.Result{}, nil
+	}
+
+	if controllerutils.IsClusterPausedOrRelocating(cd, logger) {
 		return reconcile.Result{}, nil
 	}
 
@@ -291,7 +391,11 @@ func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.R
 		logger.WithError(err).Error("unable to get REST config")
 		return reconcile.Result{}, err
 	}
-	resourceHelper, err := r.resourceHelperBuilder(restConfig, logger)
+
+	// If this cluster carries the fake annotation we will fake out all helper communication with it.
+	fakeCluster := controllerutils.IsFakeCluster(cd)
+
+	resourceHelper, err := r.resourceHelperBuilder(restConfig, fakeCluster, logger)
 	if err != nil {
 		log.WithError(err).Error("cannot create helper")
 		return reconcile.Result{}, err
@@ -311,6 +415,7 @@ func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.R
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "could not create ClusterSync")
 			return reconcile.Result{}, err
 		}
+		recobsrv.SetOutcome(hivemetrics.ReconcileOutcomeClusterSyncCreated)
 		// requeue immediately so that we reconcile soon after the ClusterSync is created
 		return reconcile.Result{Requeue: true}, nil
 	case err != nil:
@@ -344,6 +449,7 @@ func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.R
 	if needToDoFullReapply {
 		logger.Info("need to reapply all syncsets")
 	}
+	recobsrv.SetOutcome(hivemetrics.ReconcileOutcomeFullSync)
 
 	// Apply SyncSets
 	syncStatusesForSyncSets, syncSetsNeedRequeue := r.applySyncSets(
@@ -375,7 +481,7 @@ func (r *ReconcileClusterSync) Reconcile(request reconcile.Request) (reconcile.R
 
 	// Set clusterSync.Status.FirstSyncSetsSuccessTime
 	syncStatuses := append(syncStatusesForSyncSets, syncStatusesForSelectorSyncSets...)
-	if len(syncStatuses) > 0 && clusterSync.Status.FirstSuccessTime == nil {
+	if clusterSync.Status.FirstSuccessTime == nil {
 		r.setFirstSuccessTime(syncStatuses, cd, clusterSync, logger)
 	}
 
@@ -471,6 +577,10 @@ func (r *ReconcileClusterSync) applySyncSets(
 		if syncSet.GetSpec().ResourceApplyMode == hivev1.SyncResourceApplyMode {
 			newSyncStatus.ResourcesToDelete = resourcesApplied
 		}
+		if syncSet.GetSpec().ResourceApplyMode == hivev1.UpsertResourceApplyMode && len(oldSyncStatus.ResourcesToDelete) > 0 {
+			logger.Infof("resource apply mode is %v but there are resources to delete in clustersync status", hivev1.UpsertResourceApplyMode)
+			oldSyncStatus.ResourcesToDelete = nil
+		}
 		if err != nil {
 			newSyncStatus.Result = hiveintv1alpha1.FailureSyncSetResult
 			newSyncStatus.FailureMessage = err.Error()
@@ -537,7 +647,11 @@ func (r *ReconcileClusterSync) applySyncSets(
 				// For non-selector SyncSets we have a more accurate startTime, either ClusterDeployment installedTimestamp
 				// or SyncSet creationTimestamp.
 				logger.WithField("applyTime", applyTime).Debug("observed first successful apply of SyncSet for cluster")
-				metricTimeToApplySyncSet.Observe(applyTime)
+				if syncSetGroup, ok := syncSet.AsMetaObject().GetAnnotations()[constants.SyncSetMetricsGroupAnnotation]; ok && syncSetGroup != "" {
+					metricTimeToApplySyncSet.WithLabelValues(syncSetGroup).Observe(applyTime)
+				} else {
+					metricTimeToApplySyncSet.WithLabelValues("none").Observe(applyTime)
+				}
 			}
 		}
 
@@ -946,6 +1060,11 @@ func (r *ReconcileClusterSync) setFirstSuccessTime(syncStatuses []hiveintv1alpha
 			lastSuccessTime = status.FirstSuccessTime
 		}
 	}
+	// When len(syncStatuses) == 0, meaning there are no syncsets which apply to the cluster, we will use now as the last success time
+	if len(syncStatuses) == 0 {
+		now := metav1.Now()
+		lastSuccessTime = &now
+	}
 	clusterSync.Status.FirstSuccessTime = lastSuccessTime
 	allSyncSetsAppliedDuration := lastSuccessTime.Time.Sub(cd.Status.InstalledTimestamp.Time)
 	logger.Infof("observed syncsets applied duration: %v seconds", allSyncSetsAppliedDuration.Seconds())
