@@ -12,6 +12,7 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/util/flowcontrol"
 	"k8s.io/client-go/util/workqueue"
 
@@ -25,6 +26,7 @@ import (
 	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
 
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
@@ -48,6 +50,10 @@ const (
 	// avoid a false positive when the node status is checked too
 	// soon after the cluster is ready
 	nodeCheckWaitTime = 4 * time.Minute
+
+	// hibernateAfterSyncSetsNotApplied is the amount of time to wait
+	// before hibernating when SyncSets have not been applied
+	hibernateAfterSyncSetsNotApplied = 10 * time.Minute
 )
 
 var (
@@ -124,18 +130,10 @@ func AddToManager(mgr manager.Manager, r *hibernationReconciler, concurrentRecon
 
 // Reconcile syncs a single ClusterDeployment
 func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result reconcile.Result, returnErr error) {
-	start := time.Now()
-	cdLog := r.logger.WithFields(log.Fields{
-		"controller":        ControllerName,
-		"clusterDeployment": request.NamespacedName.String(),
-	})
-
+	cdLog := controllerutils.BuildControllerLogger(ControllerName, "clusterDeployment", request.NamespacedName)
 	cdLog.Info("reconciling cluster deployment")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		cdLog.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, cdLog)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the ClusterDeployment instance
 	cd := &hivev1.ClusterDeployment{}
@@ -165,11 +163,34 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 	shouldHibernate := cd.Spec.PowerState == hivev1.HibernatingClusterPowerState
 	hibernatingCondition := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition)
 
-	// Signal a problem if we should be hibernating or have requested hibernate after and the cluster does not support it.
+	// Signal a problem if we should be hibernating or have requested hibernate after and the cluster does not support it or
+	// SyncSets have not yet been applied.
 	if shouldHibernate || cd.Spec.HibernateAfter != nil {
-		if supported, msg := r.canHibernate(cd); !supported {
+		if supported, msg := r.hibernationSupported(cd); !supported {
 			return r.setHibernatingCondition(cd, hivev1.UnsupportedHibernationReason, msg, corev1.ConditionFalse, cdLog)
 		}
+
+		// Determine if SyncSets have been applied
+		clusterSync := &hiveintv1alpha1.ClusterSync{}
+		err := r.Get(context.Background(), types.NamespacedName{Namespace: cd.Namespace, Name: cd.Name}, clusterSync)
+		if err != nil {
+			return reconcile.Result{}, fmt.Errorf("could not get ClusterSync: %v", err)
+		}
+		// SyncSets have not been applied
+		if clusterSync.Status.FirstSuccessTime == nil {
+			// Allow hibernation (do not set condition) if hibernateAfterSyncSetsNotApplied duration has passed since cluster
+			// installed and syncsets still not applied
+			if cd.Status.InstalledTimestamp != nil && time.Now().Sub(cd.Status.InstalledTimestamp.Time) < hibernateAfterSyncSetsNotApplied {
+				return r.setHibernatingCondition(cd, hivev1.SyncSetsNotAppliedReason, "Cluster SyncSets have not been applied", corev1.ConditionFalse, cdLog)
+			}
+		}
+	}
+
+	// Clear any lingering unsupported hibernation condition
+	if hibernatingCondition != nil && hibernatingCondition.Reason == hivev1.UnsupportedHibernationReason {
+		if supported, msg := r.hibernationSupported(cd); supported {
+			return r.setHibernatingCondition(cd, hivev1.RunningHibernationReason, msg, corev1.ConditionFalse, cdLog)
+		}
 	}
 
 	// Check if HibernateAfter is set, and if the cluster has been in running state for longer than this duration, put it to sleep.
@@ -228,7 +249,7 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 			return reconcile.Result{}, nil
 		}
 		switch hibernatingCondition.Reason {
-		case hivev1.StoppingHibernationReason, hivev1.HibernatingHibernationReason:
+		case hivev1.StoppingHibernationReason, hivev1.HibernatingHibernationReason, hivev1.FailedToStartHibernationReason:
 			return r.startMachines(cd, cdLog)
 		case hivev1.ResumingHibernationReason:
 			return r.checkClusterResumed(cd, cdLog)
@@ -236,13 +257,14 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 		return reconcile.Result{}, nil
 	}
 
-	if hibernatingCondition == nil || hibernatingCondition.Status == corev1.ConditionFalse || hibernatingCondition.Reason == hivev1.ResumingHibernationReason {
+	if hibernatingCondition == nil || hibernatingCondition.Status == corev1.ConditionFalse ||
+		hibernatingCondition.Reason == hivev1.ResumingHibernationReason ||
+		(cd.Spec.PowerState == hivev1.HibernatingClusterPowerState && hibernatingCondition.Reason == hivev1.FailedToStartHibernationReason) {
 		return r.stopMachines(cd, cdLog)
 	}
 	if hibernatingCondition.Reason == hivev1.StoppingHibernationReason {
 		return r.checkClusterStopped(cd, false, cdLog)
 	}
-
 	return reconcile.Result{}, nil
 }
 
@@ -255,7 +277,12 @@ func (r *hibernationReconciler) startMachines(cd *hivev1.ClusterDeployment, logg
 	logger.Info("Resuming cluster")
 	if err := actuator.StartMachines(cd, r.Client, logger); err != nil {
 		msg := fmt.Sprintf("Failed to start machines: %v", err)
-		return r.setHibernatingCondition(cd, hivev1.FailedToStartHibernationReason, msg, corev1.ConditionTrue, logger)
+		result, condErr := r.setHibernatingCondition(cd, hivev1.FailedToStartHibernationReason, msg, corev1.ConditionTrue, logger)
+		if condErr != nil {
+			return reconcile.Result{}, condErr
+		}
+		// Return the error starting machines so we get requeue + backoff
+		return result, err
 	}
 	return r.setHibernatingCondition(cd, hivev1.ResumingHibernationReason, "Starting cluster machines", corev1.ConditionTrue, logger)
 }
@@ -324,7 +351,7 @@ func (r *hibernationReconciler) checkClusterResumed(cd *hivev1.ClusterDeployment
 	return r.setHibernatingCondition(cd, hivev1.RunningHibernationReason, "All machines are started and nodes are ready", corev1.ConditionFalse, logger)
 }
 
-func (r *hibernationReconciler) setHibernatingCondition(cd *hivev1.ClusterDeployment, reason, message string, status corev1.ConditionStatus, logger log.FieldLogger) (reconcile.Result, error) {
+func (r *hibernationReconciler) setHibernatingCondition(cd *hivev1.ClusterDeployment, reason, message string, status corev1.ConditionStatus, logger log.FieldLogger) (result reconcile.Result, returnErr error) {
 	changed := false
 	if status == corev1.ConditionFalse && controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition) == nil {
 		now := metav1.Now()
@@ -347,12 +374,24 @@ func (r *hibernationReconciler) setHibernatingCondition(cd *hivev1.ClusterDeploy
 			controllerutils.UpdateConditionIfReasonOrMessageChange,
 		)
 	}
+
+	if reason == hivev1.SyncSetsNotAppliedReason {
+		defer func() {
+			expiry := cd.Status.InstalledTimestamp.Time.Add(hibernateAfterSyncSetsNotApplied)
+			requeueAfter := time.Until(expiry)
+			logger.Infof("cluster will reconcile due to syncsets not applied in: %v", requeueAfter)
+			result.RequeueAfter = requeueAfter
+			result.Requeue = true
+			returnErr = errors.New(message)
+		}()
+	}
+
 	if changed {
 		if err := r.Status().Update(context.TODO(), cd); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "Failed to update hibernating condition")
 			return reconcile.Result{}, errors.Wrap(err, "failed to update hibernating condition")
 		}
-		logger.Info("Hibernating condition updated on cluster deployment.")
+		logger.WithField("reason", reason).Info("Hibernating condition updated on cluster deployment.")
 	}
 	return reconcile.Result{}, nil
 }
@@ -366,7 +405,7 @@ func (r *hibernationReconciler) getActuator(cd *hivev1.ClusterDeployment) Hibern
 	return nil
 }
 
-func (r *hibernationReconciler) canHibernate(cd *hivev1.ClusterDeployment) (bool, string) {
+func (r *hibernationReconciler) hibernationSupported(cd *hivev1.ClusterDeployment) (bool, string) {
 	if r.getActuator(cd) == nil {
 		return false, "Unsupported platform: no actuator to handle it"
 	}
@@ -381,7 +420,7 @@ func (r *hibernationReconciler) canHibernate(cd *hivev1.ClusterDeployment) (bool
 	if version.LT(minimumClusterVersion) {
 		return false, fmt.Sprintf("Unsupported version, need version %s or greater", minimumClusterVersion.String())
 	}
-	return true, ""
+	return true, "Hibernation capable"
 }
 
 func (r *hibernationReconciler) nodesReady(cd *hivev1.ClusterDeployment, remoteClient client.Client, logger log.FieldLogger) (bool, error) {
