@@ -25,10 +25,12 @@ import (
 	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
 
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/controller/hibernation/mock"
 	"github.com/openshift/hive/pkg/remoteclient"
 	remoteclientmock "github.com/openshift/hive/pkg/remoteclient/mock"
 	testcd "github.com/openshift/hive/pkg/test/clusterdeployment"
+	testcs "github.com/openshift/hive/pkg/test/clustersync"
 	testgeneric "github.com/openshift/hive/pkg/test/generic"
 )
 
@@ -45,6 +47,7 @@ func TestReconcile(t *testing.T) {
 	corev1.AddToScheme(scheme)
 	batchv1.AddToScheme(scheme)
 	hivev1.AddToScheme(scheme)
+	hiveintv1alpha1.AddToScheme(scheme)
 	machineapi.AddToScheme(scheme)
 
 	cdBuilder := testcd.FullBuilder(namespace, cdName, scheme).Options(
@@ -52,18 +55,24 @@ func TestReconcile(t *testing.T) {
 		testcd.WithClusterVersion("4.4.9"),
 	)
 	o := clusterDeploymentOptions{}
+	csBuilder := testcs.FullBuilder(namespace, cdName, scheme).Options(
+		testcs.WithFirstSuccessTime(time.Now().Add(-10 * time.Hour)),
+	)
 
 	tests := []struct {
 		name           string
 		cd             *hivev1.ClusterDeployment
+		cs             *hiveintv1alpha1.ClusterSync
 		setupActuator  func(actuator *mock.MockHibernationActuator)
 		setupCSRHelper func(helper *mock.MockcsrHelper)
 		setupRemote    func(builder *remoteclientmock.MockBuilder)
 		validate       func(t *testing.T, cd *hivev1.ClusterDeployment)
+		expectError    bool
 	}{
 		{
 			name: "cluster deleted",
 			cd:   cdBuilder.GenericOptions(testgeneric.Deleted()).Options(o.shouldHibernate).Build(),
+			cs:   csBuilder.Build(),
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				if getHibernatingCondition(cd) != nil {
 					t.Errorf("not expecting hibernating condition")
@@ -73,6 +82,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "cluster not installed",
 			cd:   cdBuilder.Options(o.notInstalled, o.shouldHibernate).Build(),
+			cs:   csBuilder.Build(),
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				require.Nil(t, getHibernatingCondition(cd))
 			},
@@ -80,6 +90,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "start hibernating, older version",
 			cd:   cdBuilder.Options(o.shouldHibernate, testcd.WithClusterVersion("4.3.11")).Build(),
+			cs:   csBuilder.Build(),
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				cond := getHibernatingCondition(cd)
 				require.NotNil(t, cond)
@@ -87,9 +98,36 @@ func TestReconcile(t *testing.T) {
 				assert.Equal(t, hivev1.UnsupportedHibernationReason, cond.Reason)
 			},
 		},
+		{
+			name: "start hibernating, syncsets not applied",
+			cd:   cdBuilder.Options(o.shouldHibernate, testcd.InstalledTimestamp(time.Now())).Build(),
+			cs:   csBuilder.Options(testcs.WithNoFirstSuccessTime()).Build(),
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionFalse, cond.Status)
+				assert.Equal(t, hivev1.SyncSetsNotAppliedReason, cond.Reason)
+			},
+			expectError: true,
+		},
+		{
+			name: "start hibernating, syncsets not applied but 10 minutes have passed since cd install",
+			cd:   cdBuilder.Options(o.shouldHibernate, testcd.InstalledTimestamp(time.Now().Add(-15*time.Minute))).Build(),
+			cs:   csBuilder.Options(testcs.WithNoFirstSuccessTime()).Build(),
+			setupActuator: func(actuator *mock.MockHibernationActuator) {
+				actuator.EXPECT().StopMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+			},
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionTrue, cond.Status)
+				assert.Equal(t, hivev1.StoppingHibernationReason, cond.Reason)
+			},
+		},
 		{
 			name: "start hibernating",
 			cd:   cdBuilder.Options(o.shouldHibernate).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().StopMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
 			},
@@ -103,6 +141,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "fail to stop machines",
 			cd:   cdBuilder.Options(o.shouldHibernate).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().StopMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(fmt.Errorf("error"))
 			},
@@ -116,6 +155,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "stopping, machines have stopped",
 			cd:   cdBuilder.Options(o.shouldHibernate, o.stopping).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
 			},
@@ -129,6 +169,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "stopping, machines have not stopped",
 			cd:   cdBuilder.Options(o.shouldHibernate, o.stopping).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(false, nil)
 			},
@@ -139,9 +180,31 @@ func TestReconcile(t *testing.T) {
 				assert.Equal(t, hivev1.StoppingHibernationReason, cond.Reason)
 			},
 		},
+		{
+			name: "stopping after MachinesFailedToStart",
+			cd: cdBuilder.Options(o.shouldHibernate).Build(
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+					Reason: hivev1.FailedToStartHibernationReason,
+				},
+				)),
+			cs: csBuilder.Build(),
+			setupActuator: func(actuator *mock.MockHibernationActuator) {
+				// Ensure we try to stop machines in this state (bugfix)
+				actuator.EXPECT().StopMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+			},
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionTrue, cond.Status)
+				assert.Equal(t, hivev1.StoppingHibernationReason, cond.Reason)
+			},
+		},
 		{
 			name: "start resuming",
 			cd:   cdBuilder.Options(o.hibernating).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().StartMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
 			},
@@ -155,9 +218,11 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "fail to start machines",
 			cd:   cdBuilder.Options(o.hibernating).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().StartMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(fmt.Errorf("error"))
 			},
+			expectError: true,
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				cond := getHibernatingCondition(cd)
 				require.NotNil(t, cond)
@@ -165,9 +230,31 @@ func TestReconcile(t *testing.T) {
 				assert.Equal(t, hivev1.FailedToStartHibernationReason, cond.Reason)
 			},
 		},
+		{
+			name: "starting machines have already failed to start",
+			cd: cdBuilder.Options(o.resuming).Build(
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+					Reason: hivev1.FailedToStartHibernationReason,
+				},
+				)),
+			cs: csBuilder.Build(),
+			setupActuator: func(actuator *mock.MockHibernationActuator) {
+				// Call will succeed which should clear the FailedToStart reason:
+				actuator.EXPECT().StartMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+			},
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionTrue, cond.Status)
+				assert.Equal(t, hivev1.ResumingHibernationReason, cond.Reason)
+			},
+		},
 		{
 			name: "starting, machines have not started",
 			cd:   cdBuilder.Options(o.resuming).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(false, nil)
 			},
@@ -181,6 +268,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "starting, machines running, nodes ready",
 			cd:   cdBuilder.Options(o.resuming).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
 			},
@@ -198,6 +286,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "starting, machines running, unready node",
 			cd:   cdBuilder.Options(o.resuming).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
 			},
@@ -217,6 +306,7 @@ func TestReconcile(t *testing.T) {
 		{
 			name: "starting, machines running, unready node, csrs to approve",
 			cd:   cdBuilder.Options(o.resuming).Build(),
+			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
 				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
 			},
@@ -240,6 +330,17 @@ func TestReconcile(t *testing.T) {
 				assert.Equal(t, hivev1.ResumingHibernationReason, cond.Reason)
 			},
 		},
+		{
+			name: "previously unsupported hibernation, now supported",
+			cd:   cdBuilder.Options(o.unsupported, testcd.WithHibernateAfter(8*time.Hour)).Build(),
+			cs:   csBuilder.Build(),
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, hivev1.RunningHibernationReason, cond.Reason)
+				assert.Equal(t, "Hibernation capable", cond.Message)
+			},
+		},
 	}
 
 	for _, test := range tests {
@@ -259,7 +360,7 @@ func TestReconcile(t *testing.T) {
 				test.setupCSRHelper(mockCSRHelper)
 			}
 			actuators = []HibernationActuator{mockActuator}
-			c := fake.NewFakeClientWithScheme(scheme, test.cd)
+			c := fake.NewFakeClientWithScheme(scheme, test.cd, test.cs)
 
 			reconciler := hibernationReconciler{
 				Client: c,
@@ -272,7 +373,12 @@ func TestReconcile(t *testing.T) {
 			_, err := reconciler.Reconcile(reconcile.Request{
 				NamespacedName: types.NamespacedName{Namespace: namespace, Name: cdName},
 			})
-			assert.Nil(t, err)
+
+			if test.expectError {
+				assert.Error(t, err, "expected error from reconcile")
+			} else {
+				assert.NoError(t, err, "expected no error from reconcile")
+			}
 			if test.validate != nil {
 				cd := &hivev1.ClusterDeployment{}
 				err := c.Get(context.TODO(), client.ObjectKey{Namespace: namespace, Name: cdName}, cd)
@@ -292,18 +398,24 @@ func TestHibernateAfter(t *testing.T) {
 	scheme := runtime.NewScheme()
 	corev1.AddToScheme(scheme)
 	hivev1.AddToScheme(scheme)
+	hiveintv1alpha1.AddToScheme(scheme)
 
 	cdBuilder := testcd.FullBuilder(namespace, cdName, scheme).Options(
 		testcd.Installed(),
 		testcd.WithClusterVersion("4.4.9"),
 	)
 	o := clusterDeploymentOptions{}
+	csBuilder := testcs.FullBuilder(namespace, cdName, scheme).Options(
+		testcs.WithFirstSuccessTime(time.Now().Add(-10 * time.Hour)),
+	)
 
 	tests := []struct {
 		name          string
 		setupActuator func(actuator *mock.MockHibernationActuator)
 		cd            *hivev1.ClusterDeployment
+		cs            *hiveintv1alpha1.ClusterSync
 
+		expectError             bool
 		expectRequeueAfter      time.Duration
 		expectedPowerState      hivev1.ClusterPowerState
 		expectedConditionReason string
@@ -313,6 +425,7 @@ func TestHibernateAfter(t *testing.T) {
 			cd: cdBuilder.Build(
 				testcd.WithHibernateAfter(8*time.Hour),
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs:                 csBuilder.Build(),
 			expectedPowerState: hivev1.HibernatingClusterPowerState,
 		},
 		{
@@ -321,6 +434,7 @@ func TestHibernateAfter(t *testing.T) {
 				testcd.WithHibernateAfter(8*time.Hour),
 				testcd.WithClusterVersion("4.3.11"),
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs:                      csBuilder.Build(),
 			expectedPowerState:      "",
 			expectedConditionReason: hivev1.UnsupportedHibernationReason,
 		},
@@ -330,6 +444,9 @@ func TestHibernateAfter(t *testing.T) {
 				testcd.WithHibernateAfter(8*time.Hour),
 				testcd.WithClusterVersion("4.3.11"),
 				testcd.InstalledTimestamp(time.Now().Add(-3*time.Hour))),
+			cs: csBuilder.Options(
+				testcs.WithFirstSuccessTime(time.Now().Add(-3 * time.Hour)),
+			).Build(),
 			expectedPowerState:      "",
 			expectedConditionReason: hivev1.UnsupportedHibernationReason,
 		},
@@ -338,6 +455,7 @@ func TestHibernateAfter(t *testing.T) {
 			cd: cdBuilder.Build(
 				testcd.WithHibernateAfter(12*time.Hour),
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs:                 csBuilder.Build(),
 			expectRequeueAfter: 2 * time.Hour,
 			expectedPowerState: "",
 		},
@@ -347,6 +465,7 @@ func TestHibernateAfter(t *testing.T) {
 				testcd.WithHibernateAfter(8*time.Hour),
 				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 9*time.Hour)),
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs:                 csBuilder.Build(),
 			expectedPowerState: hivev1.HibernatingClusterPowerState,
 		},
 		{
@@ -355,6 +474,7 @@ func TestHibernateAfter(t *testing.T) {
 				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 6*time.Hour)),
 				testcd.WithHibernateAfter(20*time.Hour),
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs:                 csBuilder.Build(),
 			expectRequeueAfter: 14 * time.Hour,
 			expectedPowerState: "",
 		},
@@ -368,9 +488,45 @@ func TestHibernateAfter(t *testing.T) {
 				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour)),
 				testcd.WithCondition(hibernatingCondition(corev1.ConditionTrue, hivev1.ResumingHibernationReason, 8*time.Hour)),
 				o.shouldRun),
+			cs:                 csBuilder.Build(),
 			expectedPowerState: hivev1.RunningClusterPowerState,
 			expectRequeueAfter: stateCheckInterval,
 		},
+		{
+			name: "cluster due for hibernate but syncsets not applied",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(8*time.Minute),
+				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 8*time.Minute)),
+				testcd.InstalledTimestamp(time.Now().Add(-8*time.Minute))),
+			cs: csBuilder.Options(
+				testcs.WithNoFirstSuccessTime(),
+			).Build(),
+			expectError:        true,
+			expectedPowerState: "",
+			expectRequeueAfter: time.Duration(time.Minute * 2),
+		},
+		{
+			name: "cluster due for hibernate, syncsets not applied but 10 minutes have passed since cd install",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(8*time.Hour),
+				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 9*time.Hour)),
+				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs: csBuilder.Options(
+				testcs.WithNoFirstSuccessTime(),
+			).Build(),
+			expectedPowerState: hivev1.HibernatingClusterPowerState,
+		},
+		{
+			name: "cluster due for hibernate, syncsets successfully applied",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(8*time.Hour),
+				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 9*time.Hour)),
+				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs: csBuilder.Options(
+				testcs.WithFirstSuccessTime(time.Now()),
+			).Build(),
+			expectedPowerState: hivev1.HibernatingClusterPowerState,
+		},
 	}
 
 	for _, test := range tests {
@@ -385,7 +541,7 @@ func TestHibernateAfter(t *testing.T) {
 			mockBuilder := remoteclientmock.NewMockBuilder(ctrl)
 			mockCSRHelper := mock.NewMockcsrHelper(ctrl)
 			actuators = []HibernationActuator{mockActuator}
-			c := fake.NewFakeClientWithScheme(scheme, test.cd)
+			c := fake.NewFakeClientWithScheme(scheme, test.cd, test.cs)
 
 			reconciler := hibernationReconciler{
 				Client: c,
@@ -399,14 +555,18 @@ func TestHibernateAfter(t *testing.T) {
 				NamespacedName: types.NamespacedName{Namespace: namespace, Name: cdName},
 			})
 
+			if test.expectError {
+				assert.Error(t, err, "expected error from reconcile")
+			} else {
+				assert.NoError(t, err, "expected no error from reconcile")
+			}
+
 			// Need to do fuzzy requeue after matching
-			if assert.NoError(t, err, "error reconciling") {
-				if test.expectRequeueAfter == 0 {
-					assert.Zero(t, result.RequeueAfter)
-				} else {
-					assert.GreaterOrEqual(t, result.RequeueAfter.Seconds(), (test.expectRequeueAfter - 10*time.Second).Seconds(), "requeue after too small")
-					assert.LessOrEqual(t, result.RequeueAfter.Seconds(), (test.expectRequeueAfter + 10*time.Second).Seconds(), "request after too large")
-				}
+			if test.expectRequeueAfter == 0 {
+				assert.Zero(t, result.RequeueAfter)
+			} else {
+				assert.GreaterOrEqual(t, result.RequeueAfter.Seconds(), (test.expectRequeueAfter - 10*time.Second).Seconds(), "requeue after too small")
+				assert.LessOrEqual(t, result.RequeueAfter.Seconds(), (test.expectRequeueAfter + 10*time.Second).Seconds(), "request after too large")
 			}
 
 			cd := &hivev1.ClusterDeployment{}
@@ -460,6 +620,13 @@ func (*clusterDeploymentOptions) resuming(cd *hivev1.ClusterDeployment) {
 		Status: corev1.ConditionTrue,
 	})
 }
+func (*clusterDeploymentOptions) unsupported(cd *hivev1.ClusterDeployment) {
+	cd.Status.Conditions = append(cd.Status.Conditions, hivev1.ClusterDeploymentCondition{
+		Type:   hivev1.ClusterHibernatingCondition,
+		Status: corev1.ConditionFalse,
+		Reason: hivev1.UnsupportedHibernationReason,
+	})
+}
 
 func getHibernatingCondition(cd *hivev1.ClusterDeployment) *hivev1.ClusterDeploymentCondition {
 	for i := range cd.Status.Conditions {
