@@ -77,15 +77,6 @@ var (
 		Help: "Total number of SyncSetsInstances referencing non-selector SyncSets that have not successfully applied all resources/patches/secrets.",
 	})
 
-	// MetricClusterDeploymentProvisionUnderwaySeconds is a prometheus metric for the number of seconds
-	// between when a still provisioning cluster was created and now.
-	MetricClusterDeploymentProvisionUnderwaySeconds = prometheus.NewGaugeVec(
-		prometheus.GaugeOpts{
-			Name: "hive_cluster_deployment_provision_underway_seconds",
-			Help: "Length of time a cluster has been provisioning.",
-		},
-		[]string{"cluster_deployment", "namespace", "cluster_type"},
-	)
 	// MetricClusterDeploymentDeprovisioningUnderwaySeconds is a prometheus metric for the number of seconds
 	// between when a still deprovisioning cluster was created and now.
 	MetricClusterDeploymentDeprovisioningUnderwaySeconds = prometheus.NewGaugeVec(
@@ -96,19 +87,33 @@ var (
 		},
 		[]string{"cluster_deployment", "namespace", "cluster_type"},
 	)
-	// MetricControllerReconcileTime tracks the length of time our reconcile loops take. controller-runtime
+	// metricControllerReconcileTime tracks the length of time our reconcile loops take. controller-runtime
 	// technically tracks this for us, but due to bugs currently also includes time in the queue, which leads to
 	// extremely strange results. For now, track our own metric.
-	MetricControllerReconcileTime = prometheus.NewHistogramVec(
+	metricControllerReconcileTime = prometheus.NewHistogramVec(
 		prometheus.HistogramOpts{
 			Name:    "hive_controller_reconcile_seconds",
 			Help:    "Distribution of the length of time each controllers reconcile loop takes.",
 			Buckets: []float64{0.001, 0.01, 0.1, 1, 10, 30, 60, 120},
 		},
-		[]string{"controller"},
+		[]string{"controller", "outcome"},
 	)
 )
 
+// ReconcileOutcome is used in controller "reconcile complete" log entries, and the metricControllerReconcileTime
+// above for controllers where we would like to monitor performance for different types of Reconcile outcomes. To help with
+// prometheus cardinality this set of outcomes should be kept small and only used for coarse and very high value
+// categories. Controllers must report an outcome but can report unspecified if this categorization is not needed.
+type ReconcileOutcome string
+
+const (
+	ReconcileOutcomeUnspecified        ReconcileOutcome = "unspecified"
+	ReconcileOutcomeNoOp               ReconcileOutcome = "no-op"
+	ReconcileOutcomeSkippedSync        ReconcileOutcome = "skipped-sync"
+	ReconcileOutcomeFullSync           ReconcileOutcome = "full-sync"
+	ReconcileOutcomeClusterSyncCreated ReconcileOutcome = "clustersync-created"
+)
+
 func init() {
 	metrics.Registry.MustRegister(metricClusterDeploymentsTotal)
 	metrics.Registry.MustRegister(metricClusterDeploymentsInstalledTotal)
@@ -122,9 +127,8 @@ func init() {
 	metrics.Registry.MustRegister(metricSelectorSyncSetClustersUnappliedTotal)
 	metrics.Registry.MustRegister(metricSyncSetsTotal)
 	metrics.Registry.MustRegister(metricSyncSetsUnappliedTotal)
-	metrics.Registry.MustRegister(MetricControllerReconcileTime)
+	metrics.Registry.MustRegister(metricControllerReconcileTime)
 
-	metrics.Registry.MustRegister(MetricClusterDeploymentProvisionUnderwaySeconds)
 	metrics.Registry.MustRegister(MetricClusterDeploymentDeprovisioningUnderwaySeconds)
 }
 
@@ -134,11 +138,11 @@ func Add(mgr manager.Manager) error {
 		Client:   mgr.GetClient(),
 		Interval: 2 * time.Minute,
 	}
+	metrics.Registry.MustRegister(newProvisioningUnderwayCollector(mgr.GetClient()))
 	err := mgr.Add(mc)
 	if err != nil {
 		return err
 	}
-
 	return nil
 }
 
@@ -161,13 +165,9 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 
 	// Run forever, sleep at the end:
 	wait.Until(func() {
-		start := time.Now()
 		mcLog := log.WithField("controller", "metrics")
-		defer func() {
-			dur := time.Since(start)
-			MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-			mcLog.WithField("elapsed", dur).Info("reconcile complete")
-		}()
+		recobsrv := NewReconcileObserver(ControllerName, mcLog)
+		defer recobsrv.ObserveControllerReconcileTime()
 
 		mcLog.Info("calculating metrics across all ClusterDeployments")
 		// Load all ClusterDeployments so we can accumulate facts about them.
@@ -184,18 +184,7 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 			for _, cd := range clusterDeployments.Items {
 				accumulator.processCluster(&cd)
 
-				if cd.DeletionTimestamp == nil {
-					if !cd.Spec.Installed {
-						// For installing clusters we report the seconds since
-						// cluster was created. clusterdeployment_controller should delete this for any cluster
-						// that is installed or deleted.
-						MetricClusterDeploymentProvisionUnderwaySeconds.WithLabelValues(
-							cd.Name,
-							cd.Namespace,
-							GetClusterDeploymentType(&cd)).Set(
-							time.Since(cd.CreationTimestamp.Time).Seconds())
-					}
-				} else {
+				if cd.DeletionTimestamp != nil {
 
 					// For deprovisioning clusters we report the seconds since
 					// cluster was deleted. clusterdeployment_controller should delete this
@@ -294,9 +283,6 @@ func (mc *Calculator) Start(stopCh <-chan struct{}) error {
 		}
 
 		mc.calculateSelectorSyncSetMetrics(mcLog)
-
-		elapsed := time.Since(start)
-		mcLog.WithField("elapsed", elapsed).Info("metrics calculation complete")
 	}, mc.Interval, stopCh)
 
 	return nil
@@ -581,3 +567,44 @@ func GetClusterDeploymentType(obj metav1.Object) string {
 	}
 	return hivev1.DefaultClusterType
 }
+
+// ReconcileObserver is used to track, log, and report metrics for controller reconcile time and outcome. Each
+// controller should instantiate one near the start of the reconcile loop, and defer a call to
+// ObserveControllerReconcileTime.
+type ReconcileObserver struct {
+	startTime      time.Time
+	controllerName hivev1.ControllerName
+	logger         log.FieldLogger
+	outcome        ReconcileOutcome
+}
+
+func NewReconcileObserver(controllerName hivev1.ControllerName, logger log.FieldLogger) *ReconcileObserver {
+	return &ReconcileObserver{
+		startTime:      time.Now(),
+		controllerName: controllerName,
+		logger:         logger,
+		outcome:        ReconcileOutcomeUnspecified,
+	}
+}
+
+func (ro *ReconcileObserver) ObserveControllerReconcileTime() {
+	dur := time.Since(ro.startTime)
+	metricControllerReconcileTime.WithLabelValues(string(ro.controllerName), string(ro.outcome)).Observe(dur.Seconds())
+	fields := log.Fields{"elapsedMillis": dur.Milliseconds(), "outcome": ro.outcome}
+	// Add a log field to categorize request duration into buckets. We can't easily query > in Kibana in
+	// OpenShift deployments due to logging limitations, so these constant buckets give us a small number of
+	// constant search strings we can use to identify slow reconcile loops.
+	for _, bucket := range elapsedDurationBuckets {
+		if dur >= bucket {
+			fields["elapsedMillisGT"] = bucket.Milliseconds()
+			break
+		}
+	}
+	ro.logger.WithFields(fields).Info("reconcile complete")
+}
+
+func (ro *ReconcileObserver) SetOutcome(outcome ReconcileOutcome) {
+	ro.outcome = outcome
+}
+
+var elapsedDurationBuckets = []time.Duration{2 * time.Minute, time.Minute, 30 * time.Second, 10 * time.Second, 5 * time.Second, time.Second, 0}
