@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"context"
 	"crypto/md5"
-	"encoding/hex"
 	"fmt"
 	"os"
 	"strconv"
@@ -29,7 +28,6 @@ import (
 	"github.com/openshift/hive/pkg/constants"
 	hiveconstants "github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/controller/images"
-	"github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/operator/assets"
 	"github.com/openshift/hive/pkg/operator/util"
 	"github.com/openshift/hive/pkg/resource"
@@ -43,16 +41,16 @@ const (
 	// secrets specified in HiveConfig
 	hiveAdditionalCASecret = "hive-additional-ca"
 
-	// hiveControllersConfigMapName is the name of the configmap to store the
-	// configurations like goroutines, qps, burst etc. for different hive controllers
-	hiveControllersConfigMapName = "hive-controllers-config"
-
 	// hiveConfigHashAnnotation is annotation on hivedeployment that contains
 	// the hash of the contents of the hive-controllers-config configmap
 	hiveConfigHashAnnotation = "hive.openshift.io/hiveconfig-hash"
 )
 
-func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig, recorder events.Recorder, mdConfigMap *corev1.ConfigMap) error {
+var (
+	controllersUsingReplicas = hivev1.ControllerNames{hivev1.ClustersyncControllerName}
+)
+
+func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig, recorder events.Recorder, mdConfigMap *corev1.ConfigMap, hiveControllersConfigHash string) error {
 
 	asset := assets.MustAsset("config/controllers/deployment.yaml")
 	hLog.Debug("reading deployment")
@@ -82,9 +80,9 @@ func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper
 		)
 	}
 
-	if dc := instance.Spec.DisabledControllers; len(dc) != 0 {
-		hiveContainer.Args = append(hiveContainer.Args, "--disabled-controllers", strings.Join(dc, ","))
-	}
+	// Always add clustersync to the list of disabled controllers since clustersync is running in a statefulset now.
+	disabledControllers := append(instance.Spec.DisabledControllers, "clustersync")
+	hiveContainer.Args = append(hiveContainer.Args, "--disabled-controllers", strings.Join(disabledControllers, ","))
 
 	if level := instance.Spec.LogLevel; level != "" {
 		hiveContainer.Args = append(hiveContainer.Args, "--log-level", level)
@@ -103,12 +101,34 @@ func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper
 
 	hiveNSName := getHiveNamespace(instance)
 
-	// By default we will try to gather logs on failed installs:
-	logsEnvVar := corev1.EnvVar{
-		Name:  constants.SkipGatherLogsEnvVar,
-		Value: strconv.FormatBool(instance.Spec.FailedProvisionConfig.SkipGatherLogs),
+	if instance.Spec.FailedProvisionConfig.AWS != nil {
+		awsSpec := instance.Spec.FailedProvisionConfig.AWS
+
+		// By default we will try to gather logs on failed installs:
+		awsLogsEnvVars := []corev1.EnvVar{
+			{
+				Name:  constants.InstallLogsUploadProviderEnvVar,
+				Value: constants.InstallLogsUploadProviderAWS,
+			},
+			{
+				Name:  constants.InstallLogsCredentialsSecretRefEnvVar,
+				Value: awsSpec.CredentialsSecretRef.Name,
+			},
+			{
+				Name:  constants.InstallLogsAWSRegionEnvVar,
+				Value: awsSpec.Region,
+			},
+			{
+				Name:  constants.InstallLogsAWSServiceEndpointEnvVar,
+				Value: awsSpec.ServiceEndpoint,
+			},
+			{
+				Name:  constants.InstallLogsAWSS3BucketEnvVar,
+				Value: awsSpec.Bucket,
+			},
+		}
+		hiveContainer.Env = append(hiveContainer.Env, awsLogsEnvVars...)
 	}
-	hiveContainer.Env = append(hiveContainer.Env, logsEnvVar)
 
 	if zoneCheckDNSServers := os.Getenv(dnsServersEnvVar); len(zoneCheckDNSServers) > 0 {
 		dnsServersEnvVar := corev1.EnvVar{
@@ -174,29 +194,6 @@ func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper
 		hiveDeployment.Spec.Replicas = &replicas
 	}
 
-	hiveControllersConfigMap := &corev1.ConfigMap{}
-	hiveControllersConfigMap.Name = hiveControllersConfigMapName
-	hiveControllersConfigMap.Namespace = hiveNSName
-	hiveControllersConfigMap.Data = make(map[string]string)
-
-	if instance.Spec.ControllersConfig != nil {
-		if instance.Spec.ControllersConfig.Default != nil {
-			setHiveControllersConfig(instance.Spec.ControllersConfig.Default, hiveControllersConfigMap, "default")
-		}
-		for _, controller := range instance.Spec.ControllersConfig.Controllers {
-			setHiveControllersConfig(&controller.Config, hiveControllersConfigMap, controller.Name)
-		}
-	}
-
-	result, err := util.ApplyRuntimeObjectWithGC(h, hiveControllersConfigMap, instance)
-	if err != nil {
-		hLog.WithError(err).Error("error applying hive-controllers-config configmap")
-		return err
-	}
-	hLog.WithField("result", result).Info("hive-controllers-config configmap applied")
-
-	hLog.Info("Hashing hive-controllers-config data onto a hive deployment annotation")
-	hiveControllersConfigHash := computeHiveControllersConfigHash(hiveControllersConfigMap)
 	if hiveDeployment.Spec.Template.Annotations == nil {
 		hiveDeployment.Spec.Template.Annotations = make(map[string]string, 1)
 	}
@@ -250,6 +247,7 @@ func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper
 	openshiftSpecificAssets := []string{
 		"config/rbac/hive_admin_role.yaml",
 		"config/rbac/hive_reader_role.yaml",
+		"config/rbac/hive_clusterpool_admin.yaml",
 		"config/rbac/hive_admin_role_binding.yaml",
 		"config/rbac/hive_reader_role_binding.yaml",
 	}
@@ -270,7 +268,7 @@ func (r *ReconcileHiveConfig) deployHive(hLog log.FieldLogger, h resource.Helper
 	}
 
 	hiveDeployment.Namespace = hiveNSName
-	result, err = util.ApplyRuntimeObjectWithGC(h, hiveDeployment, instance)
+	result, err := util.ApplyRuntimeObjectWithGC(h, hiveDeployment, instance)
 	if err != nil {
 		hLog.WithError(err).Error("error applying deployment")
 		return err
@@ -474,27 +472,3 @@ func (r *ReconcileHiveConfig) deleteAllSyncSetInstances(hLog log.FieldLogger) (n
 	}
 	return
 }
-
-func computeHiveControllersConfigHash(hiveControllersConfigMap *corev1.ConfigMap) string {
-	hasher := md5.New()
-	hasher.Write([]byte(fmt.Sprintf("%v", hiveControllersConfigMap.Data)))
-	return hex.EncodeToString(hasher.Sum(nil))
-}
-
-func setHiveControllersConfig(config *hivev1.ControllerConfig, hiveControllersConfigMap *corev1.ConfigMap, controllerName hivev1.ControllerName) {
-	if config.ConcurrentReconciles != nil {
-		hiveControllersConfigMap.Data[fmt.Sprintf(utils.ConcurrentReconcilesEnvVariableFormat, controllerName)] = strconv.Itoa(int(*config.ConcurrentReconciles))
-	}
-	if config.ClientQPS != nil {
-		hiveControllersConfigMap.Data[fmt.Sprintf(utils.ClientQPSEnvVariableFormat, controllerName)] = strconv.Itoa(int(*config.ClientQPS))
-	}
-	if config.ClientBurst != nil {
-		hiveControllersConfigMap.Data[fmt.Sprintf(utils.ClientBurstEnvVariableFormat, controllerName)] = strconv.Itoa(int(*config.ClientBurst))
-	}
-	if config.QueueQPS != nil {
-		hiveControllersConfigMap.Data[fmt.Sprintf(utils.QueueQPSEnvVariableFormat, controllerName)] = strconv.Itoa(int(*config.QueueQPS))
-	}
-	if config.QueueBurst != nil {
-		hiveControllersConfigMap.Data[fmt.Sprintf(utils.QueueBurstEnvVariableFormat, controllerName)] = strconv.Itoa(int(*config.QueueBurst))
-	}
-}
