@@ -320,7 +320,12 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 		}
 	}
 
-	h := resource.NewHelperFromRESTConfig(r.restConfig, hLog)
+	h, err := resource.NewHelperFromRESTConfig(r.restConfig, hLog)
+	if err != nil {
+		hLog.WithError(err).Error("error creating resource helper")
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
 
 	managedDomainsConfigMap, err := r.configureManagedDomains(hLog, instance)
 	if err != nil {
@@ -329,13 +334,27 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 		return reconcile.Result{}, err
 	}
 
-	err = r.deployHive(hLog, h, instance, recorder, managedDomainsConfigMap)
+	confighash, err := r.deployHiveControllersConfigMap(hLog, h, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error deploying controllers configmap")
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
+	err = r.deployHive(hLog, h, instance, recorder, managedDomainsConfigMap, confighash)
 	if err != nil {
 		hLog.WithError(err).Error("error deploying Hive")
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
 
+	err = r.deployClusterSync(hLog, h, instance, confighash)
+	if err != nil {
+		hLog.WithError(err).Error("error deploying ClusterSync")
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
 	// Cleanup legacy objects:
 	if err := r.cleanupLegacyObjects(hLog); err != nil {
 		return reconcile.Result{}, err
