@@ -56,6 +56,7 @@ func (r *helper) Apply(obj []byte) (ApplyResult, error) {
 		r.logger.WithError(err).Error("failed to setup apply command")
 		return "", err
 	}
+
 	err = applyOptions.Run()
 	if err != nil {
 		r.logger.WithError(err).
@@ -130,6 +131,9 @@ func (r *helper) createOnly(f cmdutil.Factory, obj []byte) (ApplyResult, error)
 	if err != nil {
 		return "", err
 	}
+	if info == nil {
+		r.logger.Debug("err getting info")
+	}
 	c, err := f.DynamicClient()
 	if err != nil {
 		return "", err
@@ -203,7 +207,8 @@ func (r *helper) setupApplyCommand(f cmdutil.Factory, obj []byte, ioStreams gene
 		return nil, nil, err
 	}
 	o.DeleteOptions = o.DeleteFlags.ToOptions(dynamicClient, o.IOStreams)
-	o.OpenAPISchema, _ = f.OpenAPISchema()
+	// Re-use the openAPISchema that should have been initialized in the constructor.
+	o.OpenAPISchema = r.openAPISchema
 	o.Validator, err = f.Validator(false)
 	if err != nil {
 		r.logger.WithError(err).Error("cannot obtain schema to validate objects from factory")
